/*  WindShape.c			
*  modeleur de forme pour ailes de traction ...
*/



/***********/

/* include */

/***********/


#include <stdlib.h>

#include <stdio.h>

#include <string.h>

#include <math.h>

#include <GL/glut.h>


#include <afx.h>		//class CString

#include <afxdlgs.h>	//class CFileDialog



#include "../commun/matrice.h"

#include "../commun/geom.h"

#include "../commun/plot.h"

#include "../commun/fichier.h"

#include "../commun/glui.h"



/**********/

/* define */

/**********/



#define AUTEUR_DATE "(Thierry Pebayle - 18/08/2002)"

#define sqr(f1) ((f1)*(f1))

#define NBP_BEZ		20	/*nombre de points pour les courbes de bezier*/



/*pour identification callback GLUI*/

#define TAILLE_PLUS_1	1

#define TAILLE_PLUS_5	2

#define TAILLE_MOINS_1	3

#define TAILLE_MOINS_5	4

#define PROF_CENT	1

#define PROF_BOUT	2



/**********************/

/* variables globales */

/**********************/



/** These are the live variables passed into GLUI ***/

float CoeffProgGeom=0.95f; /*coeff progression gomtrique*/

float EpaiRelCent=17.0;	/*epaisseur relative du profil central*/ 

int NbCaiss=20;			/*nombre de caissons au total*/



int VisuFace=0;		/*choix visu nervures(0) ou surfaces(1)*/

int ProjOrthoPers=0;	/*choix projection orthogonale ou perspective*/

float VrillageCent; /*vrillage en degres au bout*/

float VrillageBout; /*vrillage en degres au bout*/

int	VisuSymetrique=1;	/*visualise les 2 ou 1 moiti de l'aile en 3D*/

int VisuPtsSuspentes=1; /*visualise pts de suspentage*/



/*divers*/

float *PointSel;	/*ptr sur point selectionn*/

TAxe *AxeSel;		/*ptr sur axe selectionn*/

int xSouris, ySouris;		/*coordonnes fenetre de la souris*/

int VisuTousLesAxes = 0;	/*flag pour demande de dessin de tous les axes*/

int Rotation3d = OFF;		/*flag rotation 3d -> bouton gauche souris*/

int Translation3d = OFF;	/*flag translation 3d -> bouton droit souris*/

int ZoomTwist3d = OFF;		/*flag twist 3d -> bouton gauche et droit souris*/

int NbNerv=11;		/*nombre de nervures de la demi-aile*/	

float EpaiRelProfCent, EpaiRelProfBout; /*epaisseur relative des profils*/



/*fenetres OpenGL*/

int FenetreForme;	/*forme a plat*/

int FenetreDiedre;	/*diedre*/

int FenetreMorphing;/*profil evolutif*/

int FenetreVrillage;/*vrillage de l'aile*/

int FenetreEpaiRel;	/*Epaisseur relative*/

int Fenetre3d;		/*visu 3D*/



/*axes correspondant aux fenetres*/

TAxe *AxeForme;

TAxe *AxeDiedre;	

TAxe *AxeMorphing;

TAxe *AxeVrillage;

TAxe *Axe3d;		

TAxe *AxeEpaiRel;



/*points de controle = courbes de 4 pts(x,y)*/

TCourbe *CtrlNez, *CtrlFui, *CtrlA, *CtrlB, *CtrlC, *CtrlD;

TCourbe *CtrlDiedre, *CtrlMorphing, *CtrlVrillage, *CtrlEpaiRel;



/*Forme!*/

TForme *F=NULL;



/*courbes resultantes*/

TCourbe *CourbNez, *CourbFui, *CourbA, *CourbB, *CourbC, *CourbD;

TCourbe *CourbNerv, *CourbDiedre, *CourbMorphing;

TCourbe  *CourbVrillage, *CourbEpaiRel;



/*divers tableaux*/

TMatrice *IntProfCent, *ExtProfCent, *IntProfBout ,*ExtProfBout;

TMatrice *XNerv = NULL;



//char NomProfilCent[255], NomProfilBout[255];

char NomFichierForme[255];



//interface GLUI

GLUI *glui;

GLUI_StaticText *ProfCent, *ProfBout, *FicForme;



//pour display des infos de la forme

TInfoForme info;

GLUI_StaticText *textSurface, *textEnvergure, *textAllongement, *textCorde, *textLarg;



/***********************************************/

/*liste des procedures definies dans ce fichier*/

/***********************************************/



void display(void);

void reshape(int w, int h);

void motion(int x, int y);

void BoutonSouris(int button, int state, int x, int y);

void PositionneFenetres(void);

void keyboard(unsigned char key, int x, int y);

void InitFenetre(void);

void InitLumiere(void);

void CalculCourbesCtrl(void);



void CalculVue3d(void);



void ModifNbrAlveoles( int control );

void ModifTaille( int control );

void ModifProjection3d( int control );

void ModifVisu3d( int control );

void ModifVisuSymetrique( int control );

void ModifProgGeom( int control );

void ModifVrillage( int control );



void Quitter( int control );



void ChargerFichierForme( int control );

void SauverFichierForme( int control );



void ChargerFichierProfil( int control );



void ChargerFichierConfig(char *NomFicConfig);

void SauverFichierConfig(char *NomFicConfig);



void CalculTableauForme(void);





/***********/

/* MajInfo */

/***********/

void MajInfo(void)

{

	char surf[100], env[100], all[100], cord[100], larg[100];

	//maj chaine de caractre

	sprintf(surf,"\n\tSurface: plat=%2.2fm2, proj=%2.2fm2, r=%2.1f%%",

		info.surface, info.surfaceProj, info.surfaceProj/info.surface*100.0f);

	sprintf(env,"\n\tEnvergure: plat=%2.2fm, proj=%2.2fm, r=%2.1f%%",

		info.envergure, info.envergureProj, info.envergureProj/info.envergure*100.0f);

	sprintf(all,"\n\tAllongement: plat=%2.2f, proj=%2.2f, r=%2.1f%%",

		info.allongement, info.allongementProj, info.allongementProj/info.allongement*100.0f);

	sprintf(cord,"\n\tCorde mini = %2.3fm, maxi = %2.3fm", info.cordeMin, info.cordeMax);

	sprintf(larg,"\n\tLarg caisson mini = %2.3fm, maxi = %2.3fm", info.largMin, info.largMax);

	//maj interface

	textSurface->set_text(surf);

	textEnvergure->set_text(env);

	textAllongement->set_text(all);

	textCorde->set_text(cord);

	textLarg->set_text(larg);

}



/**********************/

/* CalculTableauForme */

/**********************/

void CalculTableauForme(void)

{

	float L;

	int i;



	/*reset tableau*/

	LibererMatrice(F->TabForme);

	F->TabForme = Zeros(NbNerv, 12);



	/*boucle  partir de la 1ere nervure du centre vers l'extrmit*/

	for (i=0; i<NbNerv; i++)

	{	

		//longueur nervure

		L = CourbNez->pts->t[i][1]-CourbFui->pts->t[i][1];

		F->TabForme->t[i][0] = L; 



		//epaisseur relative

		F->TabForme->t[i][1] = EpaiRelCent*CourbEpaiRel->pts->t[i][1];



		//xnez	//ynez	//znez

		F->TabForme->t[i][2] = CourbDiedre->pts->t[i][0];	

		F->TabForme->t[i][3] = CourbDiedre->pts->t[i][1];

		F->TabForme->t[i][4] = CourbNez->pts->t[i][1];	

		

		//inclinaison de la nervure par rapport a l'horizontale en radian

		if(i==0)

			F->TabForme->t[i][5] = 1.5708f; // pi/2 au centre par defaut

		else if(i==NbNerv-1)

			F->TabForme->t[i][5] = 1.5708f + 

			atan2(CourbDiedre->pts->t[i][1]-CourbDiedre->pts->t[i-1][1],

			CourbDiedre->pts->t[i][0]-CourbDiedre->pts->t[i-1][0]);

		else

			F->TabForme->t[i][5] = 1.5708f + 

			atan2(CourbDiedre->pts->t[i+1][1]-CourbDiedre->pts->t[i-1][1],

			CourbDiedre->pts->t[i+1][0]-CourbDiedre->pts->t[i-1][0]);

			//( atan2(CourbDiedre->pts->t[i+1][1]-CourbDiedre->pts->t[i][1],

			//CourbDiedre->pts->t[i+1][0]-CourbDiedre->pts->t[i][0])

			//+ atan2(CourbDiedre->pts->t[i][1]-CourbDiedre->pts->t[i-1][1],

			//CourbDiedre->pts->t[i][0]-CourbDiedre->pts->t[i-1][0]) )/2.0f;

		

		//vrillage / horizontale en radian

		F->TabForme->t[i][6] = CourbVrillage->pts->t[i][1]*DEG2RAD;



		//morphing / nervure centrale

		F->TabForme->t[i][7] = CourbMorphing->pts->t[i][1];



		//pos relative ligne A, B ,C ,D

		F->TabForme->t[i][8] = (CourbNez->pts->t[i][1]-CourbA->pts->t[i][1])*100.0f/L;

		F->TabForme->t[i][9] = (CourbNez->pts->t[i][1]-CourbB->pts->t[i][1])*100.0f/L;

		F->TabForme->t[i][10] = (CourbNez->pts->t[i][1]-CourbC->pts->t[i][1])*100.0f/L;

		F->TabForme->t[i][11] = (CourbNez->pts->t[i][1]-CourbD->pts->t[i][1])*100.0f/L;

	}

}



/************************/

/* ChargerFichierConfig */

/***********************$*/

void ChargerFichierConfig(char *NomFicConfig)

{

	int ws,hs;

	float xf,yf,wf,hf;



	int i;

	FILE *fid;

	char texte[50];

	int f[6];



	/*recupere taille de l'ecran*/

	ws = glutGet( GLUT_SCREEN_WIDTH );

	hs = glutGet( GLUT_SCREEN_HEIGHT );



	/*init tableau des no de fenetre*/

	f[0]=FenetreForme; f[1]=FenetreDiedre; f[2]=FenetreMorphing;

	f[3]=FenetreVrillage; f[4]=FenetreEpaiRel; f[5]=Fenetre3d;



	/*ouverture fichier en lecture*/

	printf("\nLecture fichier de Config: '%s' ", NomFicConfig);

	if( (fid = fopen( NomFicConfig, "rt" )) == NULL )

	{

      printf( "\nErreur ouverture fichier '%s'", NomFicConfig);

	  exit(0);

	}



	/*lecture nom du fichier de forme*/

	fscanf(fid,"%s %s",texte, NomFichierForme);



	/*boucle de lecture des positions/tailles normalises des fenetres*/

	for (i=0; i<6; i++)

	{

		glutSetWindow( f[i] );

		fscanf(fid,"%s %f %f %f %f\n",texte, &xf, &yf, &wf, &hf);

		glutPositionWindow((int)(xf*(float)ws), (int)(yf*(float)hs));

		glutReshapeWindow((int)(wf*(float)ws), (int)(hf*(float)hs));

	}

	/*fermeture fichier*/

	if(fclose(fid))

	{

		printf("\nProbleme  la fermeture du fichier");

		exit(0);

	}

	

	//lecture fichier de forme 

	F = LectureFichierForme(NomFichierForme);

	

	//met a jour matrice des points de controle

	//met a jour matrice des points de controle

	CtrlNez->pts = F->mCtrlNez;

	CtrlFui->pts = F->mCtrlFui;

	CtrlA->pts = F->mCtrlA;

	CtrlB->pts = F->mCtrlB;

	CtrlC->pts = F->mCtrlC;

	CtrlD->pts = F->mCtrlD;

	CtrlDiedre->pts = F->mCtrlDiedre;

	CtrlMorphing->pts = F->mCtrlMorphing;

	CtrlVrillage->pts = F->mCtrlVrillage;

	CtrlEpaiRel->pts = F->mCtrlEpaiRel;

	

	//maj interface GLUI

	CoeffProgGeom = F->CoeffProgGeom;

	EpaiRelCent = F->EpaiRelCent;

	NbCaiss = F->NbCaiss;

	if(NbCaiss%2==0)

		NbNerv=NbCaiss/2+1;

	else

		NbNerv=(NbCaiss+1)/2;

	VrillageCent = CtrlVrillage->pts->t[0][1];

	VrillageBout = CtrlVrillage->pts->t[3][1];

	

	//chargement profils

	LectureFichierProfil(F->NomProfilCent, &ExtProfCent, &IntProfCent);

	LectureFichierProfil(F->NomProfilBout, &ExtProfBout, &IntProfBout);

	EpaiRelProfCent=EpaisseurRelative(ExtProfCent, IntProfCent);

	EpaiRelProfBout=EpaisseurRelative(ExtProfBout, IntProfBout);

	InterpoleProfilBout(&ExtProfBout, ExtProfCent);

	InterpoleProfilBout(&IntProfBout, IntProfCent);

}



/***********************/

/* SauverFichierConfig */

/***********************/

void SauverFichierConfig(char *NomFicConfig)

{

	int x,y,w,h,ws,hs,i,f[6];

	FILE *fid;

	char* texte[6]=

	{"FENETRE_FORME","FENETRE_DIEDRE","FENETRE_MORPHING",

	"FENETRE_VRILLAGE","FENETRE_EPAISSEUR_RELATIVE","FENETRE_3D"};



	/*init tableau des no de fenetre*/

	f[0]=FenetreForme; f[1]=FenetreDiedre; f[2]=FenetreMorphing;

	f[3]=FenetreVrillage; f[4]=FenetreEpaiRel; f[5]=Fenetre3d;



	/*recupere taille de l'ecran*/

	ws = glutGet( GLUT_SCREEN_WIDTH );

	hs = glutGet( GLUT_SCREEN_HEIGHT );



	/*ouverture fichier en ecriture*/

	if( (fid = fopen( NomFicConfig, "wt" )) == NULL )

	{

      printf( "\nErreur ouverture fichier '%s'", NomFicConfig );

	  exit(0);

	}



	/*ecriture fichier de forme*/

	fprintf(fid,"FORME %s",	NomFichierForme);





	/*boucle ecriture des positions/tailles des fenetres normaliss*/

	for (i=0; i<6; i++)

	{

		glutSetWindow( f[i] );

		x = glutGet( GLUT_WINDOW_X );

		y = glutGet( GLUT_WINDOW_Y );

		w = glutGet( GLUT_WINDOW_WIDTH );

		h = glutGet( GLUT_WINDOW_HEIGHT );

		fprintf(fid,"\n%s %1.4f %1.4f %1.4f %1.4f",	texte[i],

			(float)x/(float)ws, y/(float)hs, (float)w/(float)ws, (float)h/(float)hs);

	}

	/*fermeture fichier*/

	if(fclose(fid))

	{

		printf("\nProbleme  la fermeture du fichier");

		exit(0);

	}

}



/************************/

/* ChargerFichierProfil */

/************************/

void ChargerFichierProfil( int control )

{

	CString NomFichier;

	char* PtrNomFichier;



	//ouverture boite de dialogue

	CFileDialog DlgOpen(TRUE, NULL, "*.*", OFN_OVERWRITEPROMPT, NULL, NULL);

	if (DlgOpen.DoModal()==IDOK)

	{

		NomFichier=DlgOpen.GetPathName();

		PtrNomFichier = NomFichier.GetBuffer(1);

		//test profil centre ou bout ?

		if(control==PROF_CENT)

		{

			LectureFichierProfil(PtrNomFichier, &ExtProfCent, &IntProfCent);

			EpaiRelProfCent=EpaisseurRelative(ExtProfCent, IntProfCent);

			ProfCent->set_text(PtrNomFichier);

			strcpy(F->NomProfilCent, PtrNomFichier);

		}

		else

		{

			LectureFichierProfil(PtrNomFichier, &ExtProfBout, &IntProfBout);

			EpaiRelProfBout=EpaisseurRelative(ExtProfBout, IntProfBout);

			ProfBout->set_text(PtrNomFichier);

			strcpy(F->NomProfilBout, PtrNomFichier);

		}

		InterpoleProfilBout(&ExtProfBout, ExtProfCent);

		InterpoleProfilBout(&IntProfBout, IntProfCent);

	}

	/*mise a jour axes*/

	VisuTousLesAxes=1; display();

}





/***********************/

/* ChargerFichierForme */

/***********************/

void ChargerFichierForme( int control )

{

	CString NomFichier;

	char* PtrNomFichier;

	TForme* oldF;



	//ouverture boite de dialogue

	CFileDialog DlgOpen(TRUE, NULL, "*.txt", OFN_OVERWRITEPROMPT, NULL, NULL);

	if (DlgOpen.DoModal()==IDOK)

	{

		//recupere nom de fichier

		NomFichier=DlgOpen.GetPathName();

		PtrNomFichier=NomFichier.GetBuffer(1);

		strcpy(NomFichierForme, PtrNomFichier);

		

		//lecture fichier de forme

		oldF = F;

		F = LectureFichierForme(NomFichierForme);

			

		//met a jour matrice des points de controle

		CtrlNez->pts = F->mCtrlNez;

		CtrlFui->pts = F->mCtrlFui;

		CtrlA->pts = F->mCtrlA;

		CtrlB->pts = F->mCtrlB;

		CtrlC->pts = F->mCtrlC;

		CtrlD->pts = F->mCtrlD;

		CtrlDiedre->pts = F->mCtrlDiedre;

		CtrlMorphing->pts = F->mCtrlMorphing;

		CtrlVrillage->pts = F->mCtrlVrillage;

		CtrlEpaiRel->pts = F->mCtrlEpaiRel;



		//libere memoire ancienne forme

		LibererForme(oldF);



		//maj interface GLUI

		FicForme->set_text(NomFichierForme);

		ProfCent->set_text(F->NomProfilCent);

		ProfBout->set_text(F->NomProfilBout);



		//maj parametres (live-variables)

		CoeffProgGeom = F->CoeffProgGeom;

		EpaiRelCent = F->EpaiRelCent;

		NbCaiss = F->NbCaiss;

		VrillageCent = CtrlVrillage->pts->t[0][1];

		VrillageBout = CtrlVrillage->pts->t[3][1];

		

		//chargement profils !!!

		LectureFichierProfil(F->NomProfilCent, &ExtProfCent, &IntProfCent);

		LectureFichierProfil(F->NomProfilBout, &ExtProfBout, &IntProfBout);

		EpaiRelProfCent=EpaisseurRelative(ExtProfCent, IntProfCent);

		EpaiRelProfBout=EpaisseurRelative(ExtProfBout, IntProfBout);

		InterpoleProfilBout(&ExtProfBout, ExtProfCent);

		InterpoleProfilBout(&IntProfBout, IntProfCent);



	}

	/*synchronisation variables GLUI*/

	glui->sync_live();

	/*mise a jour axes*/

	VisuTousLesAxes=1; display();

}



/**********************/

/* SauverFichierForme */

/**********************/

void SauverFichierForme( int control )

{

	CString NomFichier;

	LPTSTR PtrNomFichier;

	

	//ouverture boite de dialogue

	CFileDialog DlgOpen(FALSE, NULL, "*.txt", OFN_OVERWRITEPROMPT, NULL, NULL);

	if (DlgOpen.DoModal()==IDOK)

	{

		//recupere nom de fichier

		NomFichier=DlgOpen.GetPathName();

		PtrNomFichier = NomFichier.GetBuffer(1);



		//met a jour parametre de forme

		F->CoeffProgGeom = CoeffProgGeom;

		F->EpaiRelCent = EpaiRelCent;

		F->NbCaiss = NbCaiss;



		//ecriture fichier

		EcritureFichierForme(PtrNomFichier, F);



		//maj nom affich dans l'interface

		strcpy(NomFichierForme, PtrNomFichier);

		FicForme->set_text(NomFichierForme);

	}

}



/********************/

/* SauverFichierDXF */

/********************/

void SauverFichierDXF( int control )

{

	CString NomFichier;

	LPTSTR PtrNomFichier;

	

	//ouverture boite de dialogue

	CFileDialog DlgOpen(FALSE, NULL, "*.dxf", OFN_OVERWRITEPROMPT, NULL, NULL);

	if (DlgOpen.DoModal()==IDOK)

	{

		//recupere nom de fichier

		NomFichier=DlgOpen.GetPathName();

		PtrNomFichier = NomFichier.GetBuffer(1);



		//ecriture fichier DXF

		EcritureFichierDXF(PtrNomFichier, Axe3d);

	}

}



/*****************/

/* ModifProgGeom */

/*****************/

void ModifProgGeom( int control )

{

	/*mise a jour axes*/

	VisuTousLesAxes=1; display();

}



/*****************/

/* ModifVrillage */

/*****************/

void ModifVrillage( int control )

{

	/*modif vrillage*/

	CtrlVrillage->pts->t[0][1]=VrillageCent;

	CtrlVrillage->pts->t[3][1]=VrillageBout;

	/*mise a jour axes*/

	VisuTousLesAxes=1; display();

}



/*********************/

/* ModifProjection3d */

/*********************/

void ModifProjection3d( int control )

{

	/*test type de proj.*/

	if(ProjOrthoPers == 0)

		Axe3d->proj = PROJ_ORTHOGONALE;

	else

		Axe3d->proj = PROJ_PERSPECTIVE;

	/*retrace uniquement la vue 3d*/

	VisuAxe(Axe3d); glutSwapBuffers();

}



/***************/

/* ModifVisu3d */

/***************/

void ModifVisu3d( int control )

{

	/*recalcul et retrace la vue 3d*/

	CalculVue3d(); VisuAxe(Axe3d); glutSwapBuffers();

}



/***********************/

/* ModifVisuSymetrique */

/***********************/

void ModifVisuSymetrique( int control )

{

	/*recalcul et retrace la vue 3d*/

	CalculVue3d(); VisuAxe(Axe3d); glutSwapBuffers();

}



/***********/

/* Quitter */

/***********/

void Quitter( int control )

{

	/*sauvegarde dernire config*/

	//SauverFichierConfig("config.txt");

	/*libere mmoire axe et courbes associes*/

	LibererAxe(AxeForme);

	LibererAxe(AxeDiedre);	

	LibererAxe(AxeMorphing);

	LibererAxe(AxeVrillage);

	LibererAxe(Axe3d);		

	LibererAxe(AxeEpaiRel);

	/*libere mmoire forme*/

	/*attention les pts de controle ont deja t libr !!!*/

	/* -> on ne peut donc pas utiliser LibererForme...*/

	LibererMatrice(F->TabForme);

	free(F);

	/*libere mmoire diverses matrices*/

	LibererMatrice(IntProfCent);

	LibererMatrice(ExtProfCent);

	LibererMatrice(IntProfBout);

	LibererMatrice(ExtProfBout);

	LibererMatrice(XNerv);

	/*malgres tout a le debugger dit qu'il y a encore

	plein de truc qui traine en mmoire !? */

	/*sortie*/

	exit(0);

}



/********************/

/* ModifNbrAlveoles */

/********************/

void ModifNbrAlveoles( int control )

{

	int AncienNbNerv;

	AncienNbNerv = NbNerv;

	/*calcul du nouveau nombre de nervures*/

	if(NbCaiss%2==0) 

		NbNerv=NbCaiss/2+1;

	else

		NbNerv=(NbCaiss+1)/2;

	/*modif points de controle des axes morphing, vrillage, epai-rel*/

	CtrlMorphing->pts->t[3][0] = NbNerv-1; 

	CtrlMorphing->pts->t[2][0] += NbNerv - AncienNbNerv; 

	CtrlVrillage->pts->t[3][0] = NbNerv-1; 

	CtrlVrillage->pts->t[2][0] += NbNerv - AncienNbNerv; 

	CtrlEpaiRel->pts->t[3][0] = NbNerv-1; 

	CtrlEpaiRel->pts->t[2][0] += NbNerv - AncienNbNerv; 

	/*mise a jour tous les axes*/

	VisuTousLesAxes=1; display();

}



/***************/

/* ModifTaille */

/***************/

void ModifTaille( int control )

{

	float coef;

	int l,c;

	/*test decremente ou incremente*/

	if(control == TAILLE_MOINS_5) coef=0.95f;

	if(control == TAILLE_MOINS_1) coef=0.99f;

	if(control == TAILLE_PLUS_5) coef=1.05f;

	if(control == TAILLE_PLUS_1) coef=1.01f;

	/*modif des points de controle*/

	for(l=0; l<CtrlNez->pts->lig; l++)

	{

		for(c=0; c<CtrlNez->pts->col; c++)

		{

			CtrlNez->pts->t[l][c] *=  coef;

			CtrlFui->pts->t[l][c] *=  coef;

			CtrlA->pts->t[l][c] *=  coef;

			CtrlB->pts->t[l][c] *=  coef;

			CtrlC->pts->t[l][c] *=  coef;

			CtrlD->pts->t[l][c] *=  coef;

		}

	}

	/*mise a jour axes*/

	VisuTousLesAxes=1; display();

}





/***************/

/* CalculVue3d */

/***************/

void CalculVue3d(void)

{

	TMatrice *XExt,*YExt,*ZExt; //coordonnes 3D extrados

	TMatrice *XInt,*YInt,*ZInt; //coordonnes 3D intrados

	TMatrice *PtsSuspentes; //pour recuperer la position 3D des pts de suspentage

	

	/*par defaut destruction des courbes ou mesh de l'axe 3D*/

	LibererCourbesAxe(Axe3d); LibererMeshsAxe(Axe3d);

	

	/*calcul forme*/

	CalculTableauForme();

	CalculForme3D(F->TabForme,

		ExtProfCent, IntProfCent, ExtProfBout, IntProfBout,

		&XExt, &YExt, &ZExt, &XInt, &YInt, &ZInt);

	

	/*ajoute visu 3D de la forme*/

	AjoutForme3D(Axe3d, XExt, YExt, ZExt, XInt, YInt, ZInt, VisuFace, VisuSymetrique);



	/*ajoute points de suspentage*/

	if(VisuPtsSuspentes){

		AjoutPtsSuspentage(Axe3d, F->TabForme, IntProfCent, XExt, YExt, ZExt, XInt, YInt, ZInt,

			VisuSymetrique, &PtsSuspentes);}



	/*liberation matrices intermediaires !!!*/

	LibererMatrice(XExt); LibererMatrice(YExt); LibererMatrice(ZExt);

	LibererMatrice(XInt); LibererMatrice(YInt); LibererMatrice(ZInt);



	/*maj info de la forme*/

	CalculInfoForme(F, &info);

	MajInfo();

}





/***************/

/* CalculXNerv */

/***************/

void CalculXNerv(void)

{

	float CumulProg,Mult,LargCent;

	int i;



	/*calcul du nombre de nervures en fonctions du nombre de caissons*/

	/* et mise a jour position relative 1ere nervure*/

	if(NbCaiss%2==0) 

	{

		NbNerv=NbCaiss/2+1;

		CumulProg=0.0f;

	}

	else

	{

		NbNerv=(NbCaiss+1)/2;

		CumulProg=0.5f;

	}

	/*creation matrice XNerv*/

	LibererMatrice(XNerv);

	XNerv = CreerMatrice(NbNerv, 1);

	/*calcul cumul progression gomtrique*/

	Mult=1.0f;

	XNerv->t[0][0] = CumulProg;

	for(i=1; i<NbNerv; i++)

	{

		CumulProg += Mult; Mult *= CoeffProgGeom;

		XNerv->t[i][0] = CumulProg;

	}

	/*calcul largeur caisson central*/

	LargCent=CtrlNez->pts->t[3][0]/CumulProg;

	/*calcul position des nervures*/

	for(i=0; i<NbNerv; i++)	XNerv->t[i][0] *= LargCent;

}



/********************/

/* CalculCourbesCtrl */

/********************/

void CalculCourbesCtrl()

{

	TCourbe *CtrlCour, *CourbCour; 

	TMatrice *Bez, *XBez, *YBez;	/*courbe de bezier*/

	TMatrice *DevBez;

	TMatrice *XDiedre, *YDiedre;

	TMatrice *Xi, *Yi;

	float Ratio;

	int i,c;



	/************************/

	/*courbes de l'axe forme*/

	/************************/

	for(c=0; c<6; c++)

	{

		/*affectation ptr Courb et Ctrl courant*/

		switch(c)

		{

		case 0: CtrlCour = CtrlNez; CourbCour = CourbNez; break;

		case 1: CtrlCour = CtrlFui; CourbCour = CourbFui; break;

		case 2: CtrlCour = CtrlA; CourbCour = CourbA; break;

		case 3: CtrlCour = CtrlB; CourbCour = CourbB; break;

		case 4: CtrlCour = CtrlC; CourbCour = CourbC; break;

		case 5: CtrlCour = CtrlD; CourbCour = CourbD; break;

		default:;

		}

		/*calcul courde de bezier*/

		Bez = MonBezier(CtrlCour->pts, NBP_BEZ);

		LibererMatrice(CourbCour->pts);

		CourbCour->pts = CreerMatrice(NbNerv,2);

		/*interpolation linaire*/

		for (i=0; i<NbNerv; i++)

		{

			CourbCour->pts->t[i][0] = XNerv->t[i][0];

			CourbCour->pts->t[i][1] = InterpLinX(Bez, XNerv->t[i][0]);

		}

		LibererMatrice(Bez);

	}

	/*creation courbe pour visualiser les nervures*/

	LibererMatrice(CourbNerv->pts);

	CourbNerv->pts =CreerMatrice(NbNerv*2,2);

	for(i=0; i<NbNerv; i++)

	{

		CourbNerv->pts->t[2*i][0] = CourbNez->pts->t[i][0];

		CourbNerv->pts->t[2*i][1] = CourbNez->pts->t[i][1];

		CourbNerv->pts->t[2*i+1][0] = CourbFui->pts->t[i][0];

		CourbNerv->pts->t[2*i+1][1] = CourbFui->pts->t[i][1];

	}



	/************************/

	/*courbe de l'axe diedre*/

	/************************/



	/*calcul de la courbe de bezier*/

	Bez = MonBezier(CtrlDiedre->pts, NBP_BEZ);



	/*calcul du developp du diedre*/

	DevBez = Zeros(NBP_BEZ,1);

	for(i=0; i<NBP_BEZ-1; i++)

		DevBez->t[i+1][0] = DevBez->t[i][0] + 

			sqrt(sqr(Bez->t[i+1][0]-Bez->t[i][0])+sqr(Bez->t[i+1][1]-Bez->t[i][1]));



	/*mise a l'echelle de la courbe de bezier du diedre 

	 et des points de controle */

	Ratio = CtrlNez->pts->t[3][0] / DevBez->t[NBP_BEZ-1][0];

	for (i=0; i<NBP_BEZ; i++)

		{Bez->t[i][0]*=Ratio; Bez->t[i][1]*=Ratio; DevBez->t[i][0]*=Ratio;}

	for (i=0; i<4; i++)

		{CtrlDiedre->pts->t[i][0]*=Ratio; CtrlDiedre->pts->t[i][1]*=Ratio;} 



	/*interpolation des positions de nervure sur le diedre*/

	XDiedre=Zeros(NbNerv,1); YDiedre=Zeros(NbNerv,1);

	XBez=Zeros(NBP_BEZ,1); YBez=Zeros(NBP_BEZ,1);

	for(i=0; i<NBP_BEZ; i++)

		{XBez->t[i][0]=Bez->t[i][0]; YBez->t[i][0]=Bez->t[i][1];}

	InterpLinMat(DevBez, XBez, XNerv, XDiedre); 

	InterpLinMat(DevBez, YBez, XNerv, YDiedre);



	/*affectation courbe diedre*/

	LibererMatrice(CourbDiedre->pts);

	CourbDiedre->pts = CreerMatrice(NbNerv,2);

	for(i=0; i<NbNerv; i++)

		{CourbDiedre->pts->t[i][0]=XDiedre->t[i][0]; CourbDiedre->pts->t[i][1]=YDiedre->t[i][0];}



	/*destruction variable intermedaire*/

	LibererMatrice(XDiedre); LibererMatrice(YDiedre);

	LibererMatrice(Bez); LibererMatrice(XBez); LibererMatrice(YBez);



	/*************************************/

	/* courbe morphing/vrillage/epai-rel */

	/*************************************/

	for(c=0; c<3; c++)

	{

		/*affectation ptr Courb et Ctrl courant*/

		switch(c)

		{

		case 0: CtrlCour = CtrlMorphing; CourbCour = CourbMorphing; break;

		case 1: CtrlCour = CtrlVrillage; CourbCour = CourbVrillage; break;

		case 2: CtrlCour = CtrlEpaiRel; CourbCour = CourbEpaiRel; break;

		default:;

		}

		

		/*calcul de la courbe de bezier*/

		Bez = MonBezier(CtrlCour->pts, NBP_BEZ);

		

		/*interpolation des Y correspondants aux No de nervure*/

		Xi=Zeros(NbNerv,1); Yi=Zeros(NbNerv,1);

		XBez=Zeros(NBP_BEZ,1); YBez=Zeros(NBP_BEZ,1);

		for(i=0; i<NBP_BEZ; i++)

		{XBez->t[i][0]=Bez->t[i][0]; YBez->t[i][0]=Bez->t[i][1];}

		for(i=0; i<NbNerv; i++) Xi->t[i][0]=i;

		InterpLinMat(XBez, YBez, Xi, Yi); 

		

		/*affectation courbe morphing*/

		LibererMatrice(CourbCour->pts);

		//CourbMorphing->pts = Bez;

		CourbCour->pts = CreerMatrice(NbNerv,2);

		for(i=0; i<NbNerv-1; i++)

		{CourbCour->pts->t[i][0]=Xi->t[i][0];

		CourbCour->pts->t[i][1]=Yi->t[i][0];}

		/*correction en extremite*/

		CourbCour->pts->t[NbNerv-1][0]=CtrlCour->pts->t[3][0];

		CourbCour->pts->t[NbNerv-1][1]=CtrlCour->pts->t[3][1];



		/*destruction variable intermedaire*/

		LibererMatrice(Xi); LibererMatrice(Yi);	LibererMatrice(Bez);

		LibererMatrice(XBez); LibererMatrice(YBez);

	}

}



/****************/

/* display      */

/****************/

void display(void)
{
	{

		CalculXNerv();

		CalculCourbesCtrl();

		//if (AxeSel == Axe3d) CalculVue3d();

		/*visualise tous les axes*/

		if(VisuTousLesAxes == 1)

		{

			CalculVue3d();

			VisuAxe(AxeForme); glutSwapBuffers();

			VisuAxe(AxeDiedre);	glutSwapBuffers();

			VisuAxe(AxeMorphing); glutSwapBuffers();

			VisuAxe(AxeVrillage); glutSwapBuffers();

			VisuAxe(AxeEpaiRel); glutSwapBuffers();

			VisuAxe(Axe3d);	glutSwapBuffers();

			VisuTousLesAxes = 0;

		}

		else /*visualise uniquement l'axe slectionn*/

		{

			if (AxeSel != NULL)	

				VisuAxe(AxeSel); glutSwapBuffers();

		}

	}

	//printf("\nerreur: %d",glGetError());
}


/*****************/

/* reshape       */

/*****************/

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);

	VisuTousLesAxes=1;

	display();

	//glutPostRedisplay();

}



/*****************/

/* motion        */

/*****************/

void motion(int x, int y)

{

	float xs,ys; /*coordonnes souris dans l'axe*/

	float xmin,xmax,ymin,ymax; /*min max de l'axe selectionn*/

	int h,w; /*taille de la fenetre*/

	int i,j; /*indice pour centre ou extremite*/

	float RatioY, RatioX, DeltaX, DeltaY, OldY, DeltaYNezFui, YNezOuFui;

	int n; /*indice de boucle*/



	/*message pour test*/

	//printf("\n<%d,%d>",x,y);



	/*axe 3d -> rotation*/

	if (Rotation3d)

	{

		Axe3d->azimuth = Axe3d->azimuth + (x - xSouris);

		Axe3d->incidence = Axe3d->incidence + (y - ySouris);

		xSouris = x; ySouris = y;

	}

	

	/*axe 3d -> translation*/

	else if (Translation3d)

	{

		Axe3d->xcam = Axe3d->xcam + (float)(x - xSouris)/100.0;

		Axe3d->ycam = Axe3d->ycam - (float)(y - ySouris)/100.0;

		xSouris = x; ySouris = y;

	}

	

	/*axe 3d -> zoom*/

	else if (ZoomTwist3d)

	{

		Axe3d->twist = Axe3d->twist - (float)(x - xSouris);

		Axe3d->zcam = Axe3d->zcam + (float)(y - ySouris)/100.0;

		xSouris = x; ySouris = y;

	}

	

	/*test d'un point selectionn*/

	else if (PointSel!=NULL)

	{

		/*conversion en coordonnes repere*/

		w = glutGet(GLUT_WINDOW_WIDTH);

		h = glutGet(GLUT_WINDOW_HEIGHT);

		xmin=AxeSel->xmin; xmax=AxeSel->xmax;

		ymin=AxeSel->ymin; ymax=AxeSel->ymax;

		xs=xmin+(float)x*(xmax-xmin)/(float)w;

		ys=ymin+((float)h-(float)y)*(ymax-ymin)/h;



		/*calcul deplacement en X et Y*/

		DeltaY = ys - PointSel[1]; DeltaX = xs - PointSel[0];



		/*message pour test*/

		//printf(",<%2.3f,%2.3f>",xs,ys);

		

		/*affectation position souris au point selectionn*/

		/*et actions sur les autres points suivant le type de point...*/



		/* nez ou fuite, au centre ou en extremite*/

		if((PointSel == &CtrlFui->pts->t[0][0])

			||(PointSel == &CtrlNez->pts->t[0][0])

			||(PointSel == &CtrlFui->pts->t[3][0])

			||(PointSel == &CtrlNez->pts->t[3][0]))

		{

			/*modification des Y*/

			/*maj des indices i et j en fonction de centre ou extremite*/

			/*utilisation de ces 2 indices pour viter de rpter 2x le code suivant*/

			if ((PointSel == &CtrlFui->pts->t[0][0])

			||(PointSel == &CtrlNez->pts->t[0][0]))

			{i = 0; j = 1;} /*centre*/

			else

			{i = 3; j = 2;} /*extremite*/

			/*calcul larg au centre*/

			DeltaYNezFui=(CtrlNez->pts->t[i][1]-CtrlFui->pts->t[i][1]);

			if(DeltaYNezFui!=0.0f)

			{

				/*maj Nez Ou Fuite*/

				if(PointSel == &CtrlFui->pts->t[i][0])

				{

					YNezOuFui = CtrlNez->pts->t[i][1];

					CtrlFui->pts->t[j][1] = CtrlFui->pts->t[j][1] + (ys - CtrlFui->pts->t[i][1]);

					CtrlFui->pts->t[i][1] = ys;

					RatioY = (YNezOuFui-ys)/DeltaYNezFui;

				}

				else

				{

					YNezOuFui = CtrlFui->pts->t[i][1];

					CtrlNez->pts->t[j][1] = CtrlNez->pts->t[j][1] + (ys - CtrlNez->pts->t[i][1]);

					CtrlNez->pts->t[i][1] = ys;

					RatioY = -(YNezOuFui-ys)/DeltaYNezFui;

				}

				/*applique le meme ratio aux lignes de suspente*/

				/*ligne A*/

				OldY=CtrlA->pts->t[i][1];

				CtrlA->pts->t[i][1] = YNezOuFui + (CtrlA->pts->t[i][1]-YNezOuFui)*RatioY;

				CtrlA->pts->t[j][1] = CtrlA->pts->t[j][1] + (CtrlA->pts->t[i][1]-OldY);

				/*ligne B*/

				OldY=CtrlB->pts->t[i][1];

				CtrlB->pts->t[i][1] = YNezOuFui + (CtrlB->pts->t[i][1]-YNezOuFui)*RatioY;

				CtrlB->pts->t[j][1] = CtrlB->pts->t[j][1] + (CtrlB->pts->t[i][1]-OldY);

				/*ligne C*/

				OldY=CtrlC->pts->t[i][1];

				CtrlC->pts->t[i][1] = YNezOuFui + (CtrlC->pts->t[i][1]-YNezOuFui)*RatioY;

				CtrlC->pts->t[j][1] = CtrlC->pts->t[j][1] + (CtrlC->pts->t[i][1]-OldY);

				/*ligne D*/

				OldY=CtrlD->pts->t[i][1];

				CtrlD->pts->t[i][1] = YNezOuFui + (CtrlD->pts->t[i][1]-YNezOuFui)*RatioY;

				CtrlD->pts->t[j][1] = CtrlD->pts->t[j][1] + (CtrlD->pts->t[i][1]-OldY);

			}

			/*modification des X*/

			if( ((PointSel == &CtrlFui->pts->t[3][0])

			||(PointSel == &CtrlNez->pts->t[3][0]))

			&&(xs != 0) ) /*pour eviter une mechante division par 0*/

			{

				/*RatioX = xs / CtrlNez->pts->t[3][0];*/

				RatioX = xs / PointSel[0];

				CtrlNez->pts->t[3][0] = xs;

				CtrlFui->pts->t[3][0] = xs;

				CtrlA->pts->t[3][0] = xs;

				CtrlB->pts->t[3][0] = xs;

				CtrlD->pts->t[3][0] = xs;

				CtrlC->pts->t[3][0] = xs;

				for(n=1; n<3; n++)

				{

					CtrlNez->pts->t[n][0] = CtrlNez->pts->t[n][0]*RatioX;

					CtrlFui->pts->t[n][0] = CtrlFui->pts->t[n][0]*RatioX;

					CtrlA->pts->t[n][0] = CtrlA->pts->t[n][0]*RatioX;

					CtrlB->pts->t[n][0] = CtrlB->pts->t[n][0]*RatioX;

					CtrlC->pts->t[n][0] = CtrlC->pts->t[n][0]*RatioX;

					CtrlD->pts->t[n][0] = CtrlD->pts->t[n][0]*RatioX;

				}

			}

		}



		//lignes A->D en central

		else if(PointSel == &CtrlA->pts->t[0][0]){

			CtrlA->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}

		else if(PointSel == &CtrlB->pts->t[0][0]){

			CtrlB->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}

		else if(PointSel == &CtrlC->pts->t[0][0]){

			CtrlC->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}

		else if(PointSel == &CtrlD->pts->t[0][0]){

			CtrlD->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}

		

		//vrillage/morphing/EpaiRel/diedre en central

		//else if(PointSel == &CtrlVrillage->pts->t[0][0]){

		//	CtrlVrillage->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}

		else if(PointSel == &CtrlDiedre->pts->t[0][0]){

			CtrlDiedre->pts->t[1][1] += DeltaY; PointSel[1] = ys; PointSel[0]=0.0;}



		//lignes A->D en extremite

		else if(PointSel == &CtrlA->pts->t[3][0]){

			CtrlA->pts->t[2][1] += DeltaY; PointSel[1] = ys; PointSel[0]=CtrlNez->pts->t[3][0];}

		else if(PointSel == &CtrlB->pts->t[3][0]){

			CtrlB->pts->t[2][1] += DeltaY; PointSel[1] = ys; PointSel[0]=CtrlNez->pts->t[3][0];}

		else if(PointSel == &CtrlC->pts->t[3][0]){

			CtrlC->pts->t[2][1] += DeltaY; PointSel[1] = ys; PointSel[0]=CtrlNez->pts->t[3][0];}

		else if(PointSel == &CtrlD->pts->t[3][0]){

			CtrlD->pts->t[2][1] += DeltaY; PointSel[1] = ys; PointSel[0]=CtrlNez->pts->t[3][0];}

		

		//vrillage/morphing/EpaiRel/diedre en extremite

		//else if(PointSel == &CtrlVrillage->pts->t[3][0]){

		//	CtrlVrillage->pts->t[2][1] += DeltaY; PointSel[1] = ys; PointSel[0]=NbNerv-1;}

		else if(PointSel == &CtrlDiedre->pts->t[3][0]){

			CtrlDiedre->pts->t[2][0] += DeltaX; CtrlDiedre->pts->t[2][1] += DeltaY;

			PointSel[0] = xs; PointSel[1] = ys;}



		//autres points, sauf points centre/extrmit de morphing/epairel/vrillage

		else if( (PointSel != &CtrlMorphing->pts->t[0][0])

			&&(PointSel != &CtrlMorphing->pts->t[3][0])

			&&(PointSel != &CtrlEpaiRel->pts->t[0][0])

			&&(PointSel != &CtrlEpaiRel->pts->t[3][0])

			&&(PointSel != &CtrlVrillage->pts->t[0][0])

			&&(PointSel != &CtrlVrillage->pts->t[3][0]) )

		{

			DeltaY = ys - PointSel[1];

			PointSel[0]=xs;	PointSel[1]=ys;

		}

		

		//impose certaines limites au tangentes

		if( (PointSel == &CtrlVrillage->pts->t[2][0])

			||(PointSel == &CtrlMorphing->pts->t[2][0])

			||(PointSel == &CtrlEpaiRel->pts->t[2][0]) )

			if (PointSel[0]>NbNerv-1) PointSel[0]=NbNerv-1;	

			



		//impose position au vrillage central/extremite ...

		CtrlVrillage->pts->t[0][1]=VrillageCent;

		CtrlVrillage->pts->t[3][1]=VrillageBout;

	}

	

	/*affichage*/

	//printf("\nx=%d, y=%d, xs=%f, ys=%f, dist=%f       ",x,y,xs,ys,distmin);



	glutPostRedisplay();

	//glutSwapBuffers();

}



/****************/

/* BoutonSouris */

/****************/

void BoutonSouris(int button, int state, int x, int y)

{

	int i,imin;

	float dist,distmin;

	float xs,ys,xmin,xmax,ymin,ymax;

	int h,w;

	TCourbe* courbe;

	int Fenetre;



	/*memorise position de la souris dans la fenetre*/

	xSouris =x; ySouris = y;



	/*par defaut pas de point ni d'axe selectionn 

	et pas de rotation/translation/zoom 3d

	et ne pas redessinner tous les axes*/

	PointSel = NULL;

	AxeSel = NULL;

	VisuTousLesAxes = 0;



	/*Rotation3d = OFF;

	Translation3d = OFF;

	ZoomTwist3d = OFF;*/



	/*recupere axe correspondant  la fenetre courante*/

	Fenetre = glutGetWindow();

	if (Fenetre == FenetreForme) {AxeSel=AxeForme; /*printf("AxeForme ->");*/}

	if (Fenetre == FenetreDiedre) {AxeSel=AxeDiedre; /*printf("AxeDiedre ->");*/}

	if (Fenetre == FenetreMorphing) {AxeSel=AxeMorphing; /*printf("AxeMorphing ->");*/}

	if (Fenetre == FenetreVrillage) {AxeSel=AxeVrillage; /*printf("AxeVrillage ->");*/}

	if (Fenetre == FenetreEpaiRel) {AxeSel=AxeEpaiRel; /*printf("AxeEpaiRel ->");*/}

	if (Fenetre == Fenetre3d) {AxeSel=Axe3d; /*printf("Axe3d ->");*/}

	

	/*test debut/fin, rotation/translation/zoom pour Axe3d*/

	if (AxeSel == Axe3d)

	{

		if (state == GLUT_DOWN)

		{

			switch(button)

			{

			case GLUT_LEFT_BUTTON :

				if (Translation3d == ON) {Translation3d = OFF; ZoomTwist3d = ON;}

				else Rotation3d = ON;

				break;

			case GLUT_RIGHT_BUTTON :

				if (Rotation3d == ON) {Rotation3d = OFF; ZoomTwist3d = ON;}

				else Translation3d = ON;

				break;

			case GLUT_MIDDLE_BUTTON :

				ZoomTwist3d = ON;

				break;

			default: printf("\n type de bouton souris inconnu !!!");

			}

		}

		else /*state = GLUT_UP*/

		{

			Rotation3d = OFF;

			Translation3d = OFF;

			ZoomTwist3d = OFF;

		}

	}	



	/*autres axes = selection d'un point*/

	else if ((AxeSel != NULL)&&(button == GLUT_LEFT_BUTTON)&&(state == GLUT_DOWN))

	{

		//printf("\nbouton ->");

		/*recupere taille de la fenetre courante*/

		w = glutGet(GLUT_WINDOW_WIDTH);

		h = glutGet(GLUT_WINDOW_HEIGHT);

		

		/*conversion en coordonnes de l'axe des coordonnes souris*/

		xmin=AxeSel->xmin; xmax=AxeSel->xmax;

		ymin=AxeSel->ymin; ymax=AxeSel->ymax;

		xs=xmin+(float)x*(xmax-xmin)/(float)w;

		ys=ymin+((float)h-(float)y)*(ymax-ymin)/h;

		

		/*recherche du point le plus pret*/

		distmin=100000.0F; imin=0;

		courbe=AxeSel->Courb;

		while(courbe != NULL)

		{

			if ((courbe->type == CTRL_POINT)&&(courbe->pts != NULL)&&(courbe->points == 1))

			{

				for(i=0; i<courbe->pts->lig; i++)

				{

					dist = sqrt(sqr(xs-courbe->pts->t[i][0]) + sqr(ys-courbe->pts->t[i][1]));

					if (dist < distmin)

					{

						distmin=dist;

						PointSel=&courbe->pts->t[i][0];					

					}

				}

			}

			courbe=courbe->CourbSuiv;

		}

		//printf("dist=%f",distmin);

		if (distmin>0.5) PointSel=NULL;

	}

	/*fin de selection/deplacement d'un point*/

	else if ((AxeSel != NULL)&&(button == GLUT_LEFT_BUTTON)&&(state == GLUT_UP))

	{

		/*redessinne tous les axes*/

		VisuTousLesAxes = 1;

		display();

	}



}



/***************************/

/* positionne les fenetres */

/***************************/

void PositionneFenetres(void)

{

	int ws,hs;

	float xf,yf,wf,hf;



	int i;

	FILE *fid;

	char texte[50];

	int f[6];



	/*recupere taille de l'ecran*/

	ws = glutGet( GLUT_SCREEN_WIDTH );

	hs = glutGet( GLUT_SCREEN_HEIGHT );



	/*init tableau des no de fenetre*/

	f[0]=FenetreForme; f[1]=FenetreDiedre; f[2]=FenetreMorphing;

	f[3]=FenetreVrillage; f[4]=FenetreEpaiRel; f[5]=Fenetre3d;



	/*ouverture fichier en lecture*/

	if( (fid = fopen( "PosFenetres.txt", "rt" )) == NULL )

	{

      printf( "\nErreur ouverture fichier 'PosFenetres.txt'" );

	  exit(0);

	}

	/*boucle de lecture des positions/tailles normalises des fenetres*/

	for (i=0; i<6; i++)

	{

		glutSetWindow( f[i] );

		fscanf(fid,"%s %f %f %f %f\n",texte, &xf, &yf, &wf, &hf);

		glutPositionWindow((int)(xf*(float)ws), (int)(yf*(float)hs));

		glutReshapeWindow((int)(wf*(float)ws), (int)(hf*(float)hs));

	}

	/*fermeture fichier*/

	if(fclose(fid))

	{

		printf("\nProbleme  la fermeture du fichier");

		exit(0);

	}

}

	

/*****************/

/* keyboard      */

/*****************/

void keyboard(unsigned char key, int x, int y)

{

	switch (key)

	{

	case 27: Quitter(0); break;

	default:;

	}

}



/*****************/

/* InitFenetre   */

/*****************/

void InitFenetre(void)

{

	glEnable(GL_LINE_STIPPLE);

	glutDisplayFunc(display);

	glutReshapeFunc(reshape);

	glutKeyboardFunc (keyboard);

	glutMotionFunc(motion);

	glutMouseFunc(BoutonSouris);

}



/*****************/

/* InitLumiere   */

/*****************/

void InitLumiere(void)

{

	float mat_specular[]={1.0, 1.0, 1.0, 1.0};

	float mat_shininess[]={50.0};

	float light_position[]={0.0, 0.0, 1.0, 0.0};

	float white_light[]={1.0, 1.0, 0.0, 1.0};



	glShadeModel(GL_SMOOTH);

	/*glClearColor(0.0, 0.0, 0.0, 0.0);*/

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);

	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);



	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);

	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);



	glEnable(GL_LIGHTING);

	glEnable(GL_LIGHT0);

	glEnable(GL_DEPTH_TEST);

	glEnable(GL_NORMALIZE);

}



/*****************/

/* M A I N       */

/*****************/

int main(int argc, char** argv)
{

	//int i;



	/*message*/

	printf("\nWindShape ");

	printf(AUTEUR_DATE);

	printf("\n");



	/*init glut*/
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize (500, 500);
	glutInitWindowPosition (100, 100);

	

	/* creation  fenetre/axe/courbes Forme*/
	FenetreForme=glutCreateWindow ("Forme a plat");

	InitFenetre();

	AxeForme=CreerAxe(FenetreForme);

	CourbNez=CreerCourbe();

	CtrlNez=CreerCourbe();

	CourbFui=CreerCourbe(); CtrlFui=CreerCourbe();

	CourbA=CreerCourbe(); CtrlA=CreerCourbe(); 

	CourbB=CreerCourbe(); CtrlB=CreerCourbe(); 

	CourbC=CreerCourbe(); CtrlD=CreerCourbe();

	CourbD=CreerCourbe(); CtrlC=CreerCourbe();

	CourbNerv=CreerCourbe();



	/* creation fenetre/axe/courbes Diedre*/

	FenetreDiedre=glutCreateWindow ("Diedre");

	InitFenetre();

	AxeDiedre=CreerAxe(FenetreDiedre);

	CtrlDiedre=CreerCourbe(); CourbDiedre=CreerCourbe();



	/* creation fenetre/axe/courbes Morphing*/

	FenetreMorphing=glutCreateWindow ("Morphing des profils");

	InitFenetre();

	AxeMorphing=CreerAxe(FenetreMorphing);

	AxeMorphing->Norme = OFF;

	//AxeMorphing->XGrid = OFF;

	CtrlMorphing=CreerCourbe(); CourbMorphing=CreerCourbe();

	

	/* creation fenetre/axe/courbes Vrillage*/

	FenetreVrillage=glutCreateWindow ("Vrillage de l'aile");

	InitFenetre();

	AxeVrillage=CreerAxe(FenetreVrillage);

	AxeVrillage->Norme = OFF;

	//AxeVrillage->XGrid = OFF;

	CtrlVrillage=CreerCourbe(); CourbVrillage=CreerCourbe();

	

	/* creation fenetre/axe/courbes EpaiRel*/

	FenetreEpaiRel=glutCreateWindow ("Epaisseurs Relatives");

	InitFenetre();

	AxeEpaiRel=CreerAxe(FenetreEpaiRel);

	AxeEpaiRel->Norme = OFF;

	//AxeEpaiRel->XGrid = OFF;

	CtrlEpaiRel=CreerCourbe(); CourbEpaiRel=CreerCourbe();



	/* creation fenetre/axe/courbes Axe3d*/

	Fenetre3d=glutCreateWindow ("Visualisation 3D");

	InitFenetre(); InitLumiere();

	Axe3d=CreerAxe(Fenetre3d); Axe3d->axe3d=ON;



	/*charge fichier de config par default*/

	ChargerFichierConfig("ConfigDefaut.txt");



	/*proprietes*/

	CtrlNez->alt=ON; CtrlFui->alt=ON;

	CtrlA->alt=ON; CtrlB->alt=ON; CtrlC->alt=ON; CtrlD->alt=ON;

	CtrlVrillage->alt=ON; CtrlDiedre->alt=ON;

	CtrlMorphing->alt=ON; CtrlEpaiRel->alt=ON;

	CtrlNez->type=CTRL_POINT; CtrlFui->type=CTRL_POINT;

	CtrlA->type=CTRL_POINT; CtrlB->type=CTRL_POINT;

	CtrlC->type=CTRL_POINT; CtrlD->type=CTRL_POINT;

	CtrlVrillage->type=CTRL_POINT; CtrlDiedre->type=CTRL_POINT;

	CtrlMorphing->type=CTRL_POINT;

	CtrlEpaiRel->type=CTRL_POINT;



	/*proprietes*/

	CourbNez->points=OFF; CourbNez->symX=ON;

	CourbFui->points=OFF; CourbFui->symX=ON;

	CourbA->points=OFF; CourbA->symX=ON;

	CourbB->points=OFF; CourbB->symX=ON;

	CourbC->points=OFF; CourbC->symX=ON;

	CourbD->points=OFF; CourbD->symX=ON;

	CourbNerv->points=OFF; CourbNerv->symX=ON; CourbNerv->alt=ON;

	CourbDiedre->points=ON; CourbDiedre->symX=ON;



	/*chainage Axe -> Courbe*/

	AjoutCourbe(AxeForme, CtrlNez); AjoutCourbe(AxeForme, CourbNez);

	AjoutCourbe(AxeForme, CtrlFui); AjoutCourbe(AxeForme, CourbFui);

	AjoutCourbe(AxeForme, CtrlA); AjoutCourbe(AxeForme, CourbA);

	AjoutCourbe(AxeForme, CtrlB); AjoutCourbe(AxeForme, CourbB);

	AjoutCourbe(AxeForme, CtrlC); AjoutCourbe(AxeForme, CourbC);

	AjoutCourbe(AxeForme, CtrlD); AjoutCourbe(AxeForme, CourbD);

	AjoutCourbe(AxeForme, CourbNerv);

	AjoutCourbe(AxeDiedre, CtrlDiedre); AjoutCourbe(AxeDiedre, CourbDiedre);

	AjoutCourbe(AxeMorphing, CtrlMorphing); AjoutCourbe(AxeMorphing, CourbMorphing);

	AjoutCourbe(AxeVrillage, CtrlVrillage); AjoutCourbe(AxeVrillage, CourbVrillage);

	AjoutCourbe(AxeEpaiRel, CtrlEpaiRel); AjoutCourbe(AxeEpaiRel, CourbEpaiRel);

	

	/****************************************/

	/*         Here's the GLUI code         */

	/****************************************/

	

	/*affichage No de version et creation fenetre*/

	printf( "\nCe programme utilise GLUI version: %3.2f",

		GLUI_Master.get_version() );

	glui = GLUI_Master.create_glui( "Boite de dialogue",0,0,0);

		//glutGet( GLUT_SCREEN_HEIGHT )-200);

	

	/****** rollout fichiers *****/

	GLUI_Rollout *RolloutFichiers =	glui->add_rollout("Fichiers",false);



	/*panel profil centre*/

	GLUI_Panel *panel_profil_centre =

		glui->add_panel_to_panel(RolloutFichiers,"Profil centre" );

	/*nom profil du centre*/

	ProfCent = glui->add_statictext_to_panel( panel_profil_centre, "???" );

	ProfCent->set_text(F->NomProfilCent);

	/*bouton de chargement*/

	glui->add_column_to_panel(panel_profil_centre, false);

	GLUI_Button *bouton = glui->add_button_to_panel(

		panel_profil_centre, "Load", PROF_CENT, ChargerFichierProfil );

	bouton->set_w(10);



	/*panel profil bout*/

	GLUI_Panel *panel_profil_bout =

		glui->add_panel_to_panel(RolloutFichiers,"Profil bout" );

	/*nom profil du bout*/

	ProfBout = glui->add_statictext_to_panel( panel_profil_bout, "???" );

	ProfBout->set_text(F->NomProfilBout);

	/*boutons de chargement*/

	glui->add_column_to_panel(panel_profil_bout, false);

	bouton = glui->add_button_to_panel(

		panel_profil_bout, "Load", PROF_BOUT, ChargerFichierProfil );

	bouton->set_w(10);

	

	/*panel forme*/

	GLUI_Panel *panel_forme = glui->add_panel_to_panel(RolloutFichiers,"Forme");

	/*nom fichier forme*/

	FicForme = glui->add_statictext_to_panel( panel_forme, "???" );

	FicForme->set_text(NomFichierForme);

	glui->add_column_to_panel(panel_forme, false);

	bouton = glui->add_button_to_panel( panel_forme, "Load", 0, ChargerFichierForme );

	bouton->set_w(10);

	bouton = glui->add_button_to_panel( panel_forme, "Save", 0, SauverFichierForme );

	bouton->set_w(10);



	/*fichier DXF*/

	bouton = glui->add_button_to_panel( RolloutFichiers, "3D->DXF", 0, SauverFichierDXF );

	bouton->set_w(10);

	

	/**** rollout parametres ****/

	//glui->add_column(false);

	GLUI_Panel *RolloutParametres = glui->add_rollout( "Parametres", false);

	/*Nombre de caissons*/

	GLUI_Spinner *SpinNbCaiss  =

		glui->add_spinner_to_panel( RolloutParametres,

		"Nbr caissons", GLUI_SPINNER_INT, &(NbCaiss), 0, ModifNbrAlveoles);

	SpinNbCaiss -> set_int_limits( 4, 100 );

	/*epaisseur relative au centre*/

	GLUI_Spinner *SpinEpaiRel =

		glui->add_spinner_to_panel( RolloutParametres,

		"Epai. Rel. cent.:", GLUI_SPINNER_FLOAT, &(EpaiRelCent));

	SpinEpaiRel -> set_float_limits( 5.0, 30.0 );

	/*progression gomtrique*/

	GLUI_Spinner *SpinProgGeom =

		glui->add_spinner_to_panel( RolloutParametres,

		"Prog. geom.:", GLUI_SPINNER_FLOAT, &(CoeffProgGeom), 0, ModifProgGeom );

	SpinProgGeom -> set_float_limits( 0.9, 1.0 );



	/*panel vrillage*/

	GLUI_Panel *panel_vrillage = glui->add_panel_to_panel( RolloutParametres, "Vrillage" );

	/*vrillage au centre*/

	GLUI_Spinner *SpinVrillCent =

		glui->add_spinner_to_panel( panel_vrillage,

		"Centre:", GLUI_SPINNER_FLOAT, &VrillageCent, 0, ModifVrillage );

	SpinVrillCent -> set_float_limits( -10.0, 30.0 );

	/*vrillage au bout*/

	GLUI_Spinner *SpinVrillBout =

		glui->add_spinner_to_panel( panel_vrillage,

		"Bout:", GLUI_SPINNER_FLOAT, &VrillageBout, 0, ModifVrillage );

	SpinVrillBout -> set_float_limits( -10.0, 30.0 );



	/*panel taille*/

	GLUI_Panel *panel_taille = glui->add_panel_to_panel( RolloutParametres, "Taille" );

	/*taille -1%*/

	bouton = glui->add_button_to_panel(panel_taille, "-1%", TAILLE_MOINS_1, ModifTaille);

	bouton->set_w(10);

	/*taille -5%*/

	glui->add_column_to_panel(panel_taille,false);

	bouton = glui->add_button_to_panel(panel_taille, "-5%", TAILLE_MOINS_5, ModifTaille);

	bouton->set_w(10);

	/*taille +1%*/

	glui->add_column_to_panel(panel_taille,false);

	bouton = glui->add_button_to_panel(panel_taille, "+1%", TAILLE_PLUS_1, ModifTaille);

	bouton->set_w(10);

	/*taille +5%*/

	glui->add_column_to_panel(panel_taille,false);

	bouton = glui->add_button_to_panel(panel_taille, "+5%", TAILLE_PLUS_5, ModifTaille);

	bouton->set_w(10);

	

	/****** rollout visu ******/

	//glui->add_column(false);

	GLUI_Panel *RolloutVisu = glui->add_rollout( "Visualisation",false);

	/*visualisation nervures/surface*/

	GLUI_Panel *panel_visu = glui->add_panel_to_panel( RolloutVisu, "Type" );

	GLUI_RadioGroup *radio_visu =

		glui->add_radiogroup_to_panel(panel_visu,&VisuFace,0,ModifVisu3d);

	glui->add_radiobutton_to_group( radio_visu, "Nervures" );

	glui->add_radiobutton_to_group( radio_visu, "Surfaces" );

	/*choix projection orthogonale ou perspective*/

	GLUI_Panel *panel_proj = glui->add_panel_to_panel( RolloutVisu, "Projection" );

	GLUI_RadioGroup *radio_proj =

		glui->add_radiogroup_to_panel(panel_proj,&ProjOrthoPers,0,ModifProjection3d);

	glui->add_radiobutton_to_group( radio_proj, "Orthogonale" );

	glui->add_radiobutton_to_group( radio_proj, "Perspective" );

	/*choix visu symtriqe ou non*/

	glui->add_checkbox_to_panel(

		RolloutVisu,"symetrique",&VisuSymetrique,0,ModifVisuSymetrique);

	/*choix visu des points de suspentage*/

	glui->add_checkbox_to_panel(

		RolloutVisu,"pts suspentes",&VisuPtsSuspentes,0,ModifVisuSymetrique);



	//pour affichage info

	GLUI_Panel *RolloutInfo = glui->add_rollout( "Info forme",false);

	textSurface = glui->add_statictext_to_panel(RolloutInfo, "surface..."); 

	textEnvergure = glui->add_statictext_to_panel(RolloutInfo, "envergure..."); 

	textAllongement = glui->add_statictext_to_panel(RolloutInfo, "allongement..."); 

	textCorde = glui->add_statictext_to_panel(RolloutInfo, "corde..."); 

	textLarg = glui->add_statictext_to_panel(RolloutInfo, "largeur caisson...");



	/**** bouton sauver / info quitter ****/

	glui->add_button( "Quitter", 0, Quitter );



	/**** Link windows to GLUI, and register idle callback ******/

	glui->set_main_gfx_window( FenetreForme );



	/* We register the idle callback with GLUI, not with GLUT */

	GLUI_Master.set_glutIdleFunc( NULL );

	

	/*premiers calculs et trace des axes*/

	AxeSel = Axe3d;

	VisuTousLesAxes = 1;

	display();



	//maj info

	CalculInfoForme(F, &info);

	MajInfo();

	

	/*boucle glut*/

	glutMainLoop();

	return 0;
}
