/*************************************/

/* geom.c                            */

/* procdures de calcul gomtrique  */

/*************************************/



#include <stdio.h>

#include <conio.h>

#include <stdlib.h>

#include <math.h>

#include "matrice.h"

#include "geom.h"



#define sqr(l1) ((l1)*(l1))

#define pi	3.141592675f

#define DEG2RAD	(3.141592675f/180.0f)



/********************************************************

* res=MonBezier(tab,NbrPts,tabi)

* calcul Bezier de NbrPts selon la technique geometrique

* tab contient les 4 points ABCD selon le format

* [XA YA

*  XB YB

*  XC YC

*  XD YD]

*********************************************************/

TMatrice* MonBezier(TMatrice *tab, int NbrPts)

{

	TMatrice *res;

	float XA,YA,XB,YB,XC,YC,XD,YD,XE,YE,XF,YF,XG,YG,XH,YH,XI,YI,XJ,YJ;

	float t,pas;

	int i;



	/*allocation matrice resultat*/

	res=CreerMatrice(NbrPts,2);



	/*pour simpifier l'ecriture*/

	XA=tab->t[0][0]; YA=tab->t[0][1];

	XB=tab->t[1][0]; YB=tab->t[1][1];

	XC=tab->t[2][0]; YC=tab->t[2][1];

	XD=tab->t[3][0]; YD=tab->t[3][1];



	/*boucle remplissage*/

	pas=1.0F/(NbrPts-1.0F); t=0.0F;

	for (i=0; i<NbrPts; i++)

	{ 

		XE=XA+(XB-XA)*t;

		YE=YA+(YB-YA)*t;

		XF=XB+(XC-XB)*t;

		YF=YB+(YC-YB)*t;

		XG=XC+(XD-XC)*t;

		YG=YC+(YD-YC)*t;

		XH=XE+(XF-XE)*t;

		YH=YE+(YF-YE)*t;

		XI=XF+(XG-XF)*t;

		YI=YF+(YG-YF)*t;

		XJ=XH+(XI-XH)*t;

		YJ=YH+(YI-YH)*t;

		res->t[i][0]=XJ;

		res->t[i][1]=YJ;

		t+=pas;

	}

	return res;

}



/*****************************************************/

/* CalculVecteurNormal                               */

/* calcul vecteur BC normal au vecteur AB            */

/* direct (cot=+1) ou indirect (cot=-1)            */

/* de longeur l.                                     */

/*****************************************************/

void CalculVecteurNormal(float xa,float ya,float xb,float yb,

						 float *xc,float *yc,float l,int cote)

{

	float xab, yab, a11, a12, a21, a22, b1, b2; /*var intermdaires*/

	float delta, delta1, delta2; /*determinant principal et secondaires*/

	/*pour simplifier ecriture*/

	xab=xb-xa; yab=yb-ya;

	/*preparation matrices A et B pour resolution AX=B*/

	a11=-yab; a12=xab;

	a21=xab;  a22=yab;

	b1=l*(float)sqrt(xab*xab+yab*yab)*cote +yb*xab -xb*yab;

	b2=xb*xab +yb*yab;

	/*calcul determinants*/

	delta=a11*a22-a21*a12;

	if (delta !=0)

	{

		delta1=b1*a22-b2*a12;

		delta2=a11*b2-a21*b1;

		/*calcul resultat*/

		*xc=delta1/delta;

		*yc=delta2/delta;

	}

	else

	{

		printf("\npb: impossibilit de calcul dans la procdure 'CalculVecteurNormal'");

		*xc=0.0; *yc=0.0;

	}

}



/*****************************************************/

/* CalculContour                                     */

/* xy vecteurs colonnes ...						     */

/* res: matrice contenant le contour                 */

/* d : distance a la courbe en chaque point          */

/* cote: +1:contour gauche, -1:contour droite        */

/*****************************************************/

TMatrice* CalculContour(TMatrice* xy, TMatrice *d, int cote)

{

	int i; 

	TMatrice *res;



	/*test parametres ok*/

	/* a faire ...*/

	/*creation matrices*/

	res=CreerMatrice(xy->lig,2);

	/*calcul matrices contour xc,yc a partir du 2eme point*/

	for(i=0; i<xy->lig-1; i++)

	{

		CalculVecteurNormal(xy->t[i][0], xy->t[i][1], xy->t[i+1][0], xy->t[i+1][1],

			&(res->t[i+1][0]), &(res->t[i+1][1]), d->t[i][0], cote);

	}

	/*calcul premier point xc,yc*/

	CalculVecteurNormal(xy->t[1][0], xy->t[1][1], xy->t[0][0], xy->t[0][1],

		&(res->t[0][0]), &(res->t[0][1]), d->t[0][0], -cote);

	return res;

}



/*****************************************************/

/* CalculVecteurBissec                               */

/*****************************************************/

void CalculVecteurBissec(float x1,float y1,float x2,float y2, float x3,float y3,

						 float *x,float *y,float l,int cote)

{

	float x4,y4,x5,y5;

	float x12,y12,x23,y23;

	float a11,a12,a21,a22,b1,b2;

	float delta, delta1, delta2;

	//pour simplifier calcul

	x12=x2-x1; y12=y1-y2;

	x23=x3-x2; y23=y3-y2;

	//test pts 1,2,3 aligns

	if (x12*y23-x23*y12 == 0.0f)

	{

		CalculVecteurNormal(x1,y1,x2,y2,x,y,l,cote);

	}

	else

	{

		CalculVecteurNormal(x1,y1,x2,y2,&x4,&y4,l,cote);

		CalculVecteurNormal(x3,y3,x2,y2,&x5,&y5,l,-cote);

		/*preparation matrices A et B pour resolution AX=B*/

		a11=y12; a12=-x12;

		a21=y23;  a22=-x23;

		b1=x4*y12-y4*x12;

		b2=x5*y23-y5*x23;

		/*calcul determinants*/

		delta=a11*a22-a21*a12;

		if (delta !=0)

		{

			delta1=b1*a22-b2*a12;

			delta2=a11*b2-a21*b1;

			/*calcul resultat*/

			*x=delta1/delta;

			*y=delta2/delta;

		}

		else

		{

			printf("\npb: impossibilit de calcul dans la procdure 'CalculVecteurBissec'");

			*x=0.0; *y=0.0;

		}

	}

}

	

/*********************/

/* EpaisseurRelative */

/*********************/

float EpaisseurRelative(TMatrice* extrados, TMatrice* intrados)

{

	int i;

	float MaxExt=-1000.0, MinInt=+1000.0;

	/*calcul epaisseur relative -> a refaire car calcul approch*/

	for(i=0; i<extrados->lig; i++)

		if(MaxExt < extrados->t[i][1]) MaxExt=extrados->t[i][1];

	for(i=0; i<intrados->lig; i++)

		if(MinInt > intrados->t[i][1]) MinInt=intrados->t[i][1];

	return MaxExt-MinInt;

}



/*****************/

/* CalculForme3D */

/*****************/

void CalculForme3D(TMatrice *TabForme,

				   TMatrice *ExtProfCent, TMatrice *IntProfCent,

				   TMatrice *ExtProfBout, TMatrice *IntProfBout,

				   TMatrice **XExt, TMatrice **YExt, TMatrice **ZExt,

				   TMatrice **XInt, TMatrice **YInt, TMatrice **ZInt)

{

	float LongNerv, EpaiRel, xp,yp, xo,yo,zo, a,v,m;

	float EpaiRelProfCent, EpaiRelProfBout;

	float coeffx, coeffyCent, coeffyBout;

	int i,j;



	/*calcul epaisseur relative profil central et bout*/

	EpaiRelProfCent = EpaisseurRelative(ExtProfCent, IntProfCent);

	EpaiRelProfBout = EpaisseurRelative(ExtProfBout, IntProfBout);



	/*init matrices extrados*/

	//LibererMatrice(*XExt); LibererMatrice(*YExt); LibererMatrice(*ZExt); 

	*XExt = Zeros(TabForme->lig, ExtProfCent->lig);

	*YExt = Zeros(TabForme->lig, ExtProfCent->lig);

	*ZExt = Zeros(TabForme->lig, ExtProfCent->lig);



	/*init matrices intrados*/

	//LibererMatrice(*XInt); LibererMatrice(*YInt); LibererMatrice(*ZInt); 

	*XInt = Zeros(TabForme->lig, IntProfCent->lig);

	*YInt = Zeros(TabForme->lig, IntProfCent->lig);

	*ZInt = Zeros(TabForme->lig, IntProfCent->lig);



	/*boucle  partir de la 1ere nervure du centre vers l'extrmit*/

	for (i=0; i<TabForme->lig; i++)

	{

		//longueur nervure courante

		LongNerv = TabForme->t[i][0];

		//epaisseur relative

		EpaiRel = TabForme->t[i][1];

		//position xo,yo,zo du nez

		xo = TabForme->t[i][2];

		yo = TabForme->t[i][3];

		zo = TabForme->t[i][4];

		//inclinaison de la nervure par rapport a l'horizontale

		a = TabForme->t[i][5];

		//angle vrillage

		v = TabForme->t[i][6];

		//coeff morphing

		m = TabForme->t[i][7];

		//calcul coeffx et coeffy des points du profil en

		//fonction de l'paisseur relative et de la longueur de nervure

		coeffx = LongNerv/100.0f;

		coeffyCent = LongNerv*EpaiRel/(EpaiRelProfCent*100.0f);

		coeffyBout = LongNerv*EpaiRel/(EpaiRelProfBout*100.0f);

		/*boucle sur les points du profil en extrados*/

		for (j=0; j<ExtProfCent->lig; j++)

		{

			xp = ExtProfCent->t[j][0]*coeffx;

			yp = ExtProfCent->t[j][1]*coeffyCent*m

				+ ExtProfBout->t[j][1]*coeffyBout*(1.0f-m);

			(*XExt)->t[i][j] = xo+(yp-xp*(float)sin(v))*(float)cos(a);

			(*YExt)->t[i][j] = yo+(yp-xp*(float)sin(v))*(float)sin(a);

			(*ZExt)->t[i][j] = zo-(xp*(float)cos(v));

		}

		/*boucle sur les points du profil en intrados*/

		for (j=0; j<IntProfCent->lig; j++)

		{

			xp = IntProfCent->t[j][0]*coeffx;

			yp = IntProfCent->t[j][1]*coeffyCent*m

				+ IntProfBout->t[j][1]*coeffyBout*(1.0f-m);

			(*XInt)->t[i][j] = xo+(yp-xp*(float)sin(v))*(float)cos(a);

			(*YInt)->t[i][j] = yo+(yp-xp*(float)sin(v))*(float)sin(a);

			(*ZInt)->t[i][j] = zo-(xp*(float)cos(v));

		}

	}

}



/***********************/

/* InterpoleProfilBout */

/***********************/

void InterpoleProfilBout(TMatrice** XYBout, TMatrice* XYCent)

{

	TMatrice *X, *Y, *Xi, *Yi;

	int i;

	

	/*interpolation des Y du profil du bout avec les X du profil central*/

	Xi=Zeros(XYCent->lig,1); Yi=Zeros(XYCent->lig,1);

	X=Zeros((*XYBout)->lig,1); Y=Zeros((*XYBout)->lig,1);

	for(i=0; i<(*XYBout)->lig; i++)

	{X->t[i][0]=(*XYBout)->t[i][0]; Y->t[i][0]=(*XYBout)->t[i][1];}

	for(i=0; i<XYCent->lig; i++)

	{Xi->t[i][0]=XYCent->t[i][0];}

	InterpLinMat(X, Y, Xi, Yi); 



	/*affectation au profil du bout*/

	LibererMatrice((*XYBout));

	(*XYBout)=Zeros(XYCent->lig,2);

	for(i=0; i<XYCent->lig; i++)

	{(*XYBout)->t[i][0]=Xi->t[i][0]; (*XYBout)->t[i][1]=Yi->t[i][0];}





	/*liberation matrice intermediaire*/

	LibererMatrice(X); LibererMatrice(Y);

	LibererMatrice(Xi); LibererMatrice(Yi);

}



/**********/

/* dist3d */

/**********/

float dist3d(float x1, float y1, float z1, float x2, float y2, float z2)

{

	return (float) sqrt(sqr(x1-x2)+sqr(y1-y2)+sqr(z1-z2));

}





/*********************************************************************

* int_cer

* role:

*	retourne les points d'intersection de deux cercles

* parametres en entree:

*	coordonnees x,y du centre o1 et rayon r1 du premier cercle

*	coordonnees x,y du centre o2 et rayon r2 du second cercle

* resultat:

*	nombre de points solutions, coordonnees x,y de deux points p1 et p2

***********************************************************************/

void int_cer(

		float xo1, float yo1, float r1,

		float xo2, float yo2, float r2,

		int *nbr_sol, float *xp1, float *yp1, float *xp2, float *yp2)

{

	float normeO1O2, normeO1P, normeO2P;

	float a,b,c,A,B,C,delta;

	float x[4],y[4],distR[4];

	float distmin1, distmin2;

	int i, imin1, imin2;



	//init par defaut

	*xp1=0.0f; *yp1=0.0f; *xp2=0.0f; *yp2=0.0f; 

	normeO1O2 = (float)sqrt(sqr(xo1-xo2)+sqr(yo1-yo2));

	////////test cercles non secants ?

	if ( (normeO1O2>(r1+r2)) || (normeO1O2<fabs(r1-r2)) )

	{

		*nbr_sol=0;

	}

	////////test cercles tangents exterieurs ?

	else if (normeO1O2==(r1+r2))

	{

		*nbr_sol=1;

		*xp1=xo1+r1/(r1+r2)*(xo2-xo1);

		*yp1=yo1+r1/(r1+r2)*(yo2-yo1);

	}

	////////test cercles tangents interieurs ?

	else if (normeO1O2==fabs(r1-r2))

	{

		////////test cercles confondus ?

		if (normeO1O2==0.0f)

		{

			*nbr_sol=0;

		}

		////////cercles tangents interieurs en un point

		else 

		{

			*nbr_sol=1;

			*xp1=xo1+r1/(r1-r2)*(xo2-xo1);

			*yp1=yo1+r1/(r1-r2)*(yo2-yo1);

		}

	}

	////////cercles secants en 2 points ...

	else

	{

		*nbr_sol=2;

		////////resolution du systeme dans le repere centre sur o1

		A=xo1-xo2;

		B=yo1-yo2;

		C=r1*r1-r2*r2+A*A+B*B;

		a=4*A*A+4*B*B;

		b=2*A*C;

		c=C*C-4*B*B*r1*r1;

		////////resolution du systeme d'ordre 2: a*x^2+2*b*x+c=0

		delta=b*b-a*c;

		x[0]=(-b+(float)sqrt(delta))/a;

		x[1]=(-b-(float)sqrt(delta))/a;

		////////les quatres solutions resultantes

		x[2]=x[0];

		x[3]=x[1];

		y[0]=(float)sqrt(r1*r1-x[0]*x[0]);

		y[1]=(float)sqrt(r1*r1-x[1]*x[1]);

		y[2]=-y[0];

		y[3]=-y[1];

		////////elimination de 2 solutions

		////////remarque: repere centre sur O1

		//remarques: P point de coordonnees x[i],y[i]

		for(i=0; i<4; i++)

		{

			normeO1P = (float)sqrt( sqr(x[i]) + sqr(y[i]) );

			normeO2P = (float)sqrt( sqr(x[i]+xo1-xo2) + sqr(y[i]+yo1-yo2) );

			distR[i] = (float)sqrt( sqr(normeO1P-r1) + sqr(normeO2P-r2) );

		}

		// choix de la solution au sens des moindres carrs ...

		//test normeO1P=r1 et normeO2P=r2

		//choix des deux valeurs mini ...

		distmin1=1000000000000.0f; imin1=0;

		for(i=0; i<4; i++){

			if (distmin1 > distR[i]){

				distmin1 = distR[i]; imin1 = i;}}

		distmin2=1000000000000.0f; imin2=0;

		for(i=0; i<4; i++){

			if ( (distmin2 > distR[i]) && (imin1 != i) ){

				distmin2 = distR[i]; imin2 = i;}}

		*xp1 = x[imin1]; *yp1 = y[imin1]; 

		*xp2 = x[imin2]; *yp2 = y[imin2]; 

		////////changement de repere

		*xp1=*xp1+xo1; *yp1=*yp1+yo1;

		*xp2=*xp2+xo1; *yp2=*yp2+yo1;

	}

	//resultat=[nbr_sol xp1 yp1 xp2 yp2];

}



/*********************************************************************

* int_cer_bis

* role:

*	retourne les points d'intersection de deux cercles

* parametres en entree:

*	coordonnees x,y du centre o1 et rayon r1 du premier cercle

*	coordonnees x,y du centre o2 et rayon r2 du second cercle

* resultat:

*	nombre de points solutions, coordonnees x,y de deux points p1 et p2

***********************************************************************/

void int_cer_bis

(

		float xo1, float yo1, float r1,

		float xo2, float yo2, float r2,

		int *nbr_sol, float *xp1, float *yp1, float *xp2, float *yp2)

{

	float normeO1O2;

	float M,N,a,b,c,delta;



	//init par defaut

	*xp1=0.0f; *yp1=0.0f; *xp2=0.0f; *yp2=0.0f; 

	normeO1O2 = (float)sqrt(sqr(xo1-xo2)+sqr(yo1-yo2));

	////////test cercles non secants ?

	if ( (normeO1O2>(r1+r2)) || (normeO1O2<fabs(r1-r2)) )

	{

		*nbr_sol=0;

	}

	////////test cercles tangents exterieurs ?

	else if (normeO1O2==(r1+r2))

	{

		*nbr_sol=1;

		*xp1=xo1+r1/(r1+r2)*(xo2-xo1);

		*yp1=yo1+r1/(r1+r2)*(yo2-yo1);

	}

	////////test cercles tangents interieurs ?

	else if (normeO1O2==fabs(r1-r2))

	{

		////////test cercles confondus ?

		if (normeO1O2==0.0f)

		{

			*nbr_sol=0;

		}

		////////cercles tangents interieurs en un point

		else 

		{

			*nbr_sol=1;

			*xp1=xo1+r1/(r1-r2)*(xo2-xo1);

			*yp1=yo1+r1/(r1-r2)*(yo2-yo1);

		}

	}

	////////cercles secants en 2 points ...

	else

	{

		*nbr_sol=2;

		//on se ramene  un systme a 2 equations: une d'ordre 1 et l'autre d'ordre 2

		//  y=Mx+N

		//	(x-xo1)^2+(y-yo1)^2-r1^2=0

		// avec pour M et N:

		M=(xo1-xo2)/(yo2-yo1);

		N=(sqr(xo2)-sqr(xo1)+sqr(yo2)-sqr(yo1)-sqr(r2)+sqr(r1))/(2*(yo2-yo1));

		a=1+sqr(M);

		b=2*M*(N-yo1)-2*xo1;

		c=sqr(xo1)+sqr(N-yo1)-sqr(r1);

		////////resolution du systeme d'ordre 2: a*x^2+2*b*x+c=0

		delta=b*b-4*a*c;

		*xp1=(-b+(float)sqrt(delta))/(2*a);

		*xp2=(-b-(float)sqrt(delta))/(2*a);

		////////les quatres solutions resultantes

		*yp1=(float)(M*(*xp1)+N);

		*yp2=(float)(M*(*xp2)+N);

	}

	//resultat=[nbr_sol xp1 yp1 xp2 yp2];

}



/*******************/

/* CalculDeveloppe */

/*******************/

void CalculDeveloppe(

					 //coordonnes 3D des 2 cotes de la surface 

					 TMatrice *Xs1, TMatrice *Ys1, TMatrice *Zs1,

					 TMatrice *Xs2, TMatrice *Ys2, TMatrice *Zs2,

					 //coordonnes 2D du dvelopp

					 TMatrice **Xb1, TMatrice **Yb1,

					 TMatrice **Xb2, TMatrice **Yb2)

					 

{

	float diag, dX, dY, X1, Y1, X2, Y2, Scal1, Scal2;

	int i, N;

	//pour rotation

	TMatrice *Thb1, *Thb2, *Rb1, *Rb2;

	float Th;

	

	//test nb pts identique

	if (Xs1->lig != Xs2->lig)

	{

		printf("\n erreur procedure CalculDeveloppe");

		fflush(stdin); getch(); exit(0);

	}

	//init tableaux

	*Xb1=Zeros(Xs1->lig,1);	*Yb1=Zeros(Xs1->lig,1);

	*Xb2=Zeros(Xs1->lig,1);	*Yb2=Zeros(Xs1->lig,1);

	(*Yb2)->t[0][0] = dist3d(

		Xs2->t[0][0],Ys2->t[0][0],Zs2->t[0][0],

		Xs1->t[0][0],Ys1->t[0][0],Zs1->t[0][0]);

	//boucle 

	for(i=0; i<Xs1->lig-1; i++)

	{

		/////////////////////////////////////

		//calcul point suivant sur le bord 2

		/////////////////////////////////////

		diag=dist3d(

			Xs2->t[i+1][0],Ys2->t[i+1][0],Zs2->t[i+1][0],

			Xs1->t[i][0],Ys1->t[i][0],Zs1->t[i][0]);     

		dX=dist3d(

			Xs2->t[i+1][0],Ys2->t[i+1][0],Zs2->t[i+1][0],

			Xs2->t[i][0],Ys2->t[i][0],Zs2->t[i][0]);     

		//test b1(i) et b2(i) confondus ?

		if ((Xs2->t[i][0]==Xs1->t[i][0])

			&&(Ys2->t[i][0]==Ys1->t[i][0])

			&&(Zs2->t[i][0]==Zs1->t[i][0]))

		{

			(*Xb2)->t[i+1][0]=(*Xb2)->t[i][0];

			(*Yb2)->t[i+1][0]=(*Yb2)->t[i][0]+dX;

		}

		else

		{

			//test points b2->t[i][0] et b2->t[i+1][0] confondus ?

			if (dX!=0.0f)

			{

				//int_cer(

				int_cer_bis(

					(*Xb1)->t[i][0], (*Yb1)->t[i][0], diag,

					(*Xb2)->t[i][0], (*Yb2)->t[i][0],dX,

					&N, &X1, &Y1, &X2, &Y2);

				//choix du point resultat ?

				//calcul de produits scalaires ...

				if (i>0)

				{

					Scal1 =

						(X1-(*Xb2)->t[i][0])*((*Xb2)->t[i][0]-(*Xb2)->t[i-1][0])

						+(Y1-(*Yb2)->t[i][0])*((*Yb2)->t[i][0]-(*Yb2)->t[i-1][0]);

					Scal2 =

						(X2-(*Xb2)->t[i][0])*((*Xb2)->t[i][0]-(*Xb2)->t[i-1][0])

						+(Y2-(*Yb2)->t[i][0])*((*Yb2)->t[i][0]-(*Yb2)->t[i-1][0]);

					if (Scal1>Scal2)

					{

						(*Xb2)->t[i+1][0]=X1; (*Yb2)->t[i+1][0]=Y1;

					}

					else

					{

						(*Xb2)->t[i+1][0]=X2; (*Yb2)->t[i+1][0]=Y2;

					}

				}

				else //(i=0) 1er point calcule du bord 2

				{

					if (X1>X2)

					{

						(*Xb2)->t[i+1][0]=X1;

						(*Yb2)->t[i+1][0]=Y1;

					}

					else

					{

						(*Xb2)->t[i+1][0]=X2;

						(*Yb2)->t[i+1][0]=Y2;

					}

				}

			}

			else //points b2->t[i][0] et b2->t[i+1][0] confondus

			{

				(*Xb2)->t[i+1][0]=(*Xb2)->t[i][0];

				(*Yb2)->t[i+1][0]=(*Yb2)->t[i][0];

			} //test points b2->t[i][0] et b2->t[i+1][0] confondus ?   

		} //test b1->t[i][0] et b2->t[i][0] confondus ?

		

		/////////////////////////////////////

		//calcul point suivant sur le bord 1

		/////////////////////////////////////

		dY=dist3d(

			Xs2->t[i+1][0],Ys2->t[i+1][0],Zs2->t[i+1][0],

			Xs1->t[i+1][0],Ys1->t[i+1][0],Zs1->t[i+1][0]);     

		dX=dist3d(

			Xs1->t[i+1][0],Ys1->t[i+1][0],Zs1->t[i+1][0],

			Xs1->t[i][0],Ys1->t[i][0],Zs1->t[i][0]);

		//test points b1->t[i][0] et b1->t[i+1][0] confondus ?

		if (dX!=0)

		{

			//test points b1->t[i+1][0] et b2->t[i+1][0] confondus ?

			if (dY!=0)

			{

				//int_cer(

				int_cer_bis(

					(*Xb2)->t[i+1][0],(*Yb2)->t[i+1][0],dY,

					(*Xb1)->t[i][0],(*Yb1)->t[i][0],dX,

					&N, &X1, &Y1, &X2, &Y2);

				//choix du point resultat ?

				//calcul de produits scalaires ...

				if (i>1)

				{

					Scal1 =

						(X1-(*Xb1)->t[i][0])*((*Xb1)->t[i][0]-(*Xb1)->t[i-1][0])

						+(Y1-(*Yb1)->t[i][0])*((*Yb1)->t[i][0]-(*Yb1)->t[i-1][0]);

					Scal2 =

						(X2-(*Xb1)->t[i][0])*((*Xb1)->t[i][0]-(*Xb1)->t[i-1][0])

						+(Y2-(*Yb1)->t[i][0])*((*Yb1)->t[i][0]-(*Yb1)->t[i-1][0]);

					if (Scal1>Scal2)

					{

						(*Xb1)->t[i+1][0]=X1; (*Yb1)->t[i+1][0]=Y1;

					}

					else

					{

						(*Xb1)->t[i+1][0]=X2; (*Yb1)->t[i+1][0]=Y2;

					}

				}

				else //1er point calcule bord 1

				{

					if (X1>X2)

					{

						(*Xb1)->t[i+1][0]=X1;

						(*Yb1)->t[i+1][0]=Y1;

					}

					else

					{

						(*Xb1)->t[i+1][0]=X2;

						(*Yb1)->t[i+1][0]=Y2;

					}

				}

			}

			else //b1->t[i+1][0] et b2->t[i+1][0] confondus

			{

				(*Xb1)->t[i+1][0]=(*Xb2)->t[i+1][0];

				(*Yb1)->t[i+1][0]=(*Yb2)->t[i+1][0];

			} //test points b1->t[i+1][0] et b2->t[i+1][0] confondus

		}

		else

		{

			(*Xb1)->t[i+1][0]=(*Xb1)->t[i][0];

			(*Yb1)->t[i+1][0]=(*Yb1)->t[i][0];

		}//test b1->t[i][0] et b1->t[i+1][0] points confondus   

	}	//boucle point



	//////////////////////////////////////////

	// aligne extremite du bord 1 sur l'axe X

	//////////////////////////////////////////

	//passage en coordonnees polaire

	Cart2Pol(*Xb1, *Yb1, &Thb1, &Rb1);

	Cart2Pol(*Xb2, *Yb2, &Thb2, &Rb2);

	//rotation

	for(i=0; i<Thb1->lig; i++)

	{

		Th = Thb1->t[Thb1->lig-1][0];

		Thb1->t[i][0] -= Th;

		Thb2->t[i][0] -= Th;

	}

	//passage en coordonnees cartesienne

	LibererMatrice(*Xb1); LibererMatrice(*Yb1);

	LibererMatrice(*Xb2); LibererMatrice(*Yb2);

	Pol2Cart(Thb1, Rb1, Xb1, Yb1);

	Pol2Cart(Thb2, Rb2, Xb2, Yb2);

	LibererMatrice(Thb1); LibererMatrice(Rb1);

	LibererMatrice(Thb2); LibererMatrice(Rb2);

}



/*************************************************/

/* passage de coordonnes cartesiennes  polaire */

/*************************************************/

void Cart2Pol(TMatrice *X, TMatrice *Y, TMatrice **T, TMatrice **R)

{

	int i;

	*R=CreerMatrice(X->lig,1); *T=CreerMatrice(X->lig,1);

	for(i=0; i<X->lig; i++)

	{

		(*T)->t[i][0] = (float)atan2(Y->t[i][0], X->t[i][0]);

		(*R)->t[i][0] = (float)sqrt(sqr(Y->t[i][0])+sqr(X->t[i][0]));

	}

}



/*************************************************/

/* passage de coordonnes polaire  cartesiennes */

/*************************************************/

void Pol2Cart(TMatrice *T, TMatrice *R, TMatrice **X, TMatrice **Y)

{

	int i;

	*X=CreerMatrice(R->lig,1); *Y=CreerMatrice(R->lig,1);

	for(i=0; i<R->lig; i++)

	{

		(*X)->t[i][0] = (float)(R->t[i][0]*cos(T->t[i][0]));

		(*Y)->t[i][0] = (float)(R->t[i][0]*sin(T->t[i][0]));

	}

}







/*******************************/

/* intersectione de 2 vecteurs */

/*******************************/

void Inter2Vecteurs(

					float xa, float ya,

					float xb, float yb, 

					float xc, float yc,

					float xd, float yd,

					float *x, float *y)

{

	float xab, yab, xcd, ycd;

	float a11,a12,a21,a22,b1,b2,delta,delta1,delta2;

	//poursimplifier ecriture

	xab=xb-xa; yab=yb-ya;

	xcd=xd-xc; ycd=yd-yc;

	//preparation matrices A et B pour resolution AX=B

	a11=yab; a12=-xab; b1=xa*yab-ya*xab;

	a21=ycd; a22=-xcd; b2=xc*ycd-yc*xcd;

	//calcul determinants

	delta=a11*a22-a21*a12;

	if (delta !=0)

	{

		delta1=b1*a22-b2*a12;

		delta2=a11*b2-a21*b1;

		/*calcul resultat*/

		*x=delta1/delta;

		*y=delta2/delta;

	}

	else

	{

		printf("\npb: impossibilit de calcul dans la procdure 'Inter2Vecteurs'");

		*x=0.0f; *y=0.0f;

	}

}



/*******************************************************************************

*

*     Linear vorticity surface panel method for airfoils.

*

*     Adapted from Kuethe and Chow 4th Edition

*     This version by I. Kroo  2-2-87

*

*     No attempt has been made to make this particularly fast or efficient.

*

*     Inputs:

*     -------

*     XB, YB     x and y coordinates of panel edges starting at trailing edge

*                proceeding forward on lower surface, wrapping around

*                leading edge and then running back to the trailingedge.

*     alphaD     Angle of attack in degrees

*

*     Outputs:

*     --------

*     X, Y       coordinates of panel centers

*     Cp         incompressible pressure coefficient at panel center

*     

********************************************************************************/

// version originale en fortran, traduit en C et Matlab par Thierry Pbayle, 2001

// function [X,Y,Cp]=LVFoil(XB,YB,alphaD);

void LVFoil(TMatrice *XB, TMatrice *YB, float alphaD, TMatrice **X, TMatrice **Y, TMatrice **Cp )

{

	

	int n, np1, i, ip1, j;

	float alpha, SINA, COSA;

	TMatrice *S, *THETA, *SINT, *COST;

	TMatrice *CN1, *CN2, *CT1, *CT2, *AN, *AT, *RHS, *XX, *V;

	float A,B,C,D,E,F,G,P1,P2,P3,P4,P,Q;

	float ANmax;



	

	//

	//     Calculation of geometric data:

	//     ------------------------------

	

	n=XB->lig-1; // n:Number of panels

	np1 = n+1;

	

	//init matrices

	*X=Zeros(n,1); *Y=Zeros(n,1); *Cp=Zeros(n,1);

	S=Zeros(n,1); THETA=Zeros(n,1); SINT=Zeros(n,1); COST=Zeros(n,1);

	

	//calcul position centre et taille des panneaux 

	for(i=0;i<n; i++)

	{

		ip1=i+1;

		(*X)->t[i][0] = .5f * (XB->t[i][0] + XB->t[ip1][0]);

		(*Y)->t[i][0] = .5f * (YB->t[i][0] + YB->t[ip1][0]);

		S->t[i][0] = (float)sqrt( sqr(XB->t[ip1][0]-XB->t[i][0]) + sqr(YB->t[ip1][0]-YB->t[i][0]) );

		THETA->t[i][0] = (float)atan2( (YB->t[ip1][0]-YB->t[i][0]), (XB->t[ip1][0]-XB->t[i][0]) );

		SINT->t[i][0] = (float)sin(THETA->t[i][0]);

		COST->t[i][0] = (float)cos(THETA->t[i][0]);

	}	

	alpha = alphaD * DEG2RAD;

	SINA = (float)sin(alpha); 

	COSA = (float)cos(alpha);

	

	//

	//     Calculation of influence coefficients

	//     -------------------------------------

	//

	

	// init tableaux

	CN1=Zeros(n,n); CN2=Zeros(n,n); CT1=Zeros(n,n); CT2=Zeros(n,n);

	AN=Zeros(np1,np1); AT=Zeros(np1,np1);

	

	for (i=0; i<n; i++)

	{

		for (j=0; j<n; j++)

		{

			if (i==j) 

			{

				CN1->t[i][j] = -1.0;

				CN2->t[i][j] = 1.0;

				CT1->t[i][j] = 0.5*pi;

				CT2->t[i][j] = CT1->t[i][j];

			}

			else

			{

				A = -((*X)->t[i][0]-XB->t[j][0])*COST->t[j][0]

					- ((*Y)->t[i][0]-YB->t[j][0])*SINT->t[j][0];

				B = sqr((*X)->t[i][0]-XB->t[j][0]) + sqr((*Y)->t[i][0]-YB->t[j][0]);

				C = SINT->t[i][0]*COST->t[j][0]-COST->t[i][0]*SINT->t[j][0];

				D = COST->t[i][0]*COST->t[j][0]+SINT->t[i][0]*SINT->t[j][0];

				E = ((*X)->t[i][0]-XB->t[j][0])*SINT->t[j][0]

					- ((*Y)->t[i][0]-YB->t[j][0])*COST->t[j][0];

				F = (float)log( 1. + S->t[j][0]*(S->t[j][0]+2.*A)/B );

				G = (float)atan2(E*S->t[j][0], B+A*S->t[j][0]);

				P1 = 1.0f-2.0f*SINT->t[j][0]*SINT->t[j][0];

				P2 = 2.0f*SINT->t[j][0]*COST->t[j][0];

				P3 = SINT->t[i][0]*P1 - COST->t[i][0]*P2;

				P4 = COST->t[i][0]*P1 + SINT->t[i][0]*P2;

				P = ((*X)->t[i][0]-XB->t[j][0]) * P3 + ((*Y)->t[i][0]-YB->t[j][0]) * P4;

				Q = ((*X)->t[i][0]-XB->t[j][0]) * P4 - ((*Y)->t[i][0]-YB->t[j][0]) * P3;

				CN2->t[i][j] = D + ( .5f*Q*F - (A*C+D*E)*G )/S->t[j][0];

				CN1->t[i][j] = .5f*D*F + C*G - CN2->t[i][j];

				CT2->t[i][j] = C + ( .5f*P*F + (A*D-C*E)*G )/S->t[j][0];

				CT1->t[i][j] = .5f*C*F - D*G - CT2->t[i][j];

			}

		}

	}

	

	ANmax = 0.0;

	for (i=0; i<n; i++)

	{

		AN->t[i][0] = CN1->t[i][0];

		AN->t[i][n] = CN2->t[i][n-1];

		AT->t[i][0] = CT1->t[i][0];

		AT->t[i][n] = CT2->t[i][n-1];

		for (j=1; j<n; j++)

		{

			AN->t[i][j] = CN1->t[i][j] + CN2->t[i][j-1];

			AT->t[i][j] = CT1->t[i][j] + CT2->t[i][j-1];

			if(fabs(AN->t[i][j])>ANmax)

			{

				ANmax = (float)fabs(AN->t[i][j]);

			}

		}

	}

				

	//     The Kutta Condition is imposed by the relation:

	//     A*gamma(1) + A*gamma(n+1) = 0.  A would be 1 but for matrix

	//     conditioning problems.

			

	AN->t[n][0] = 1.0;

	AN->t[n][n] = 1.0;



	for(j=1; j<n; j++)

	{

		AN->t[n][j] = 0.0;

	}

				

	//

	//     Decompose and solve the system:

	//     -------------------------------

	//

	//     AN.XX = RHS

	//     avec  RHS(i) = sin( THETA(i)-alpha )

	//			

	RHS=Zeros(np1,1);

	for(i=0; i<n; i++)

	{

		RHS->t[i][0] = SINT->t[i][0]*COSA - COST->t[i][0]*SINA;

	}

	//XX=inv(AN)*RHS;

	ResolutionGauss(AN, RHS, &XX);

				

	//

	//     Compute derived quantities:

	//     ---------------------------

	//

	//     V(i) = cos( THETA(i) - alpha )

	V=Zeros(n,1);

	*Cp=Zeros(n,1);

	for (i=0; i<n; i++)

	{

		V->t[i][0] =  COST->t[i][0]*COSA + SINT->t[i][0]*SINA;

		for (j=0; j<np1; j++)

		{

			V->t[i][0] = V->t[i][0] + AT->t[i][j] * XX->t[j][0];

		}

		(*Cp)->t[i][0] = 1-V->t[i][0]*V->t[i][0];

	}



	//liberation matrices de calcul

	LibererMatrice(S); LibererMatrice(THETA); LibererMatrice(SINT); LibererMatrice(COST);

	LibererMatrice(CN1); LibererMatrice(CN2); LibererMatrice(CT1); LibererMatrice(CT2);

	LibererMatrice(AN); LibererMatrice(AT); LibererMatrice(RHS); LibererMatrice(XX);

	LibererMatrice(V);



}





/*********************************************/

/* calcul longueur dvelopp d'une courbe XY */

/*********************************************/

TMatrice* Longueur(TMatrice* x, TMatrice *y)

{

	int i; 

	TMatrice *res;

	

	res=CreerMatrice(x->lig,1);

	res->t[0][0]=0.0f;//premire valeur =0.0 

	for(i=1; i<x->lig; i++)

		res->t[i][0] = res->t[i-1][0]

		+ (float)sqrt(sqr(x->t[i][0]-x->t[i-1][0])+sqr(y->t[i][0]-y->t[i-1][0]));

	return res;

}



/*************************************/

/* calcul coordones XY d'un cercle  */

/*************************************/

TMatrice* Cercle(float xo, float yo, float rayon, int nbp)

{

	int i; 

	TMatrice *res;



	res=CreerMatrice(nbp+1,2);

	for(i=0; i<nbp; i++)

	{

		res->t[i][0] = xo + (float)cos(2*pi*i/nbp)*rayon;

		res->t[i][1] = yo + (float)sin(2*pi*i/nbp)*rayon;

	}

	res->t[nbp][0] = res->t[0][0]; res->t[nbp][1] = res->t[0][1];

	return res;

}



